using System;
using System.IO;
using System.Security;
using Microsoft.Win32;

namespace CDBurnerXP.IO
{
    public partial class PathEx
    {
        public static string GetTempRipPath() {
            string path = Path.Combine(Path.GetTempPath(), "TempRips");
            try
            {
                Directory.CreateDirectory(path);
            } catch { };
            return path;
        }

        public static string GetTempRipFile()
        {
            return GetTempRipFile(string.Empty);
        }

        public static string GetTempRipFile(string suffix)
        {
            try
            {
                string file = Path.GetTempFileName();
                string target = Path.Combine(GetTempRipPath(), Path.ChangeExtension(Path.GetFileName(file), ".wav"));
                if (!string.IsNullOrEmpty(suffix))
                {
                    target = AppendToFileName(suffix, target);
                }
                File.Move(file, target);
                return target;
            }
            catch
            {
                return Path.GetTempFileName();
            }
        }

        /// <summary>
        /// Returns the full absolute path for a given location and file.
        /// </summary>
        /// <param name="basePath">File or folder to use as base</param>
        /// <param name="file">Absolute or relative path to a file</param>
        public static string GetFullPath(string basePath, string file)
        {
            if (Path.IsPathRooted(file))
            {
                return file;
            }
            else
            {
                return Path.Combine(Path.GetDirectoryName(basePath), file);
            }
        }

        public static bool IsFolderEmpty(string sPath)
        {
            DirectoryInfo dirInfo = new DirectoryInfo(sPath);
            if (dirInfo.GetFiles().Length == 0 & dirInfo.GetDirectories().Length == 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public static bool IsShowHiddenFilesEnabled
        {
            get
            {
                try
                {
                    RegistryKey key = Registry.CurrentUser;
                    key = key.OpenSubKey(@"Software\Microsoft\Windows\CurrentVersion\Explorer\Advanced");
                    int value = Convert.ToInt16(key.GetValue("Hidden"));
                    return (value == 1);
                }
                catch
                {
                    return false;
                }
            }
        }

        public static string AppendToFileName(string append, string filename)
        {
            string path = Path.GetDirectoryName(filename);
            string ext = Path.GetExtension(filename);
            filename = Path.GetFileNameWithoutExtension(filename);
            return Path.Combine(path, filename + append + ext);
        }

        public static string ReplaceInvalidPathChars(string text)
        {
            char[] invalidChars = Path.GetInvalidFileNameChars();
            foreach (char check in invalidChars)
            {
                text = text.Replace(check, '_');
            }
            return text;
        }

        public static bool IsPathDirectory(string strPath)
        {
            try
            {
                FileInfo FileProps = new FileInfo(strPath);
                if (((int)FileProps.Attributes > -1) && (FileProps.Attributes & FileAttributes.Directory) == FileAttributes.Directory)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch (Exception)
            {
                return false;
            }
        }

        public static string GetLastPathItem(string strPath)
        {
            return strPath.Substring(strPath.LastIndexOf("\\") + 1);
        }

        public static string QualifyPath(string sPath)
        {
            if (sPath.EndsWith("\\"))
            {
                return sPath;
            }
            else
            {
                return sPath + "\\";
            }
        }

        public static bool FileBusy(string sPath)
        {
            FileStream oFileStream;
            try
            {
                oFileStream = new FileStream(sPath, FileMode.Open, FileAccess.Read);
            }
            catch (Exception ex)
            {
                if (ex is IOException)
                {
                    return true;
                }
                else if (ex is UnauthorizedAccessException)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            oFileStream.Close();
            return false;
        }

        public static bool FileBusy(string sPath, ref string sInfo)
        {
            FileStream oFileStream;
            try
            {
                oFileStream = new FileStream(sPath, FileMode.Open, FileAccess.Read);
            }
            catch (IOException ex)
            {
                sInfo = ex.Message;
                return true;
            }
            catch (UnauthorizedAccessException ex)
            {
                sInfo = ex.Message;
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
            }
            oFileStream.Close();
            return false;
        }

        /// <summary>
        /// Simple function to check for some invalid chars in a file name
        /// </summary>
        /// <param name="filename"></param>
        /// <returns>No path!</returns>
        public static bool IsInvalidFileName(string filename)
        {
            char[] invalidChars = Path.GetInvalidFileNameChars();
            foreach (char check in invalidChars)
            {
                if (filename.Contains(check.ToString()))
                {
                    return true;
                }
            }
            return false;
        }

        public static string GetFileTypeFromExt(string ext)
        {
            RegistryKey rk = Registry.ClassesRoot;
            RegistryKey exta;
            RegistryKey exta2;
            try
            {
                exta = rk.OpenSubKey(ext);
                if (exta == null)
                {
                    return "";
                }
                if (exta.GetValue("") == null)
                {
                    return "";
                }
                exta2 = rk.OpenSubKey((string)exta.GetValue(""));
                if (exta2 == null)
                {
                    return "";
                }
                return exta2.GetValue("") as string;
            }
            catch (SecurityException)
            {
                return "";
            }
        }

        public static void TryDeleteFiles(string[] sTempFilePaths)
        {
            if (sTempFilePaths == null) return;

            foreach (string file in sTempFilePaths)
            {
                TryDeleteFiles(file);
            }
        }

        public static void TryDeleteFiles(string file)
        {
            try
            {
                if (File.Exists(file))
                {
                    File.Delete(file);
                }
            }
            catch
            {
                // don't care
            }
        }

        public static string TryGetFolderPath(System.Environment.SpecialFolder folder)
        {
            // the .NET function fails, if the path is for example set to e: instead of e:\
            string result = string.Empty;

            try
            {
                result = Environment.GetFolderPath(folder);
            }
            catch (ArgumentException)
            {
                // Not so nice. Let's check the registry ourselves.
                string folderName = folder.ToString();
                try
                {
                    RegistryKey key = Registry.CurrentUser;
                    key = key.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\User Shell Folders");
                    if ((key != null) & (key.GetValue(folderName) != null))
                    {
                        result = key.GetValue(folderName).ToString().TrimEnd('\\');
                    }
                }
                catch (Exception)
                {
                    // if we can't access registry, we have no choice but to return an empty string
                }
            }

            return result;
        }

        public static long TryGetFileSize(string path)
        {
            if (string.IsNullOrEmpty(path)) return 0;

            try
            {
                if (!File.Exists(path)) return 0;

                FileInfo info = new FileInfo(path);
                return info.Length;
            }
            catch (Exception)
            {
                return 0;
            }
        }
    }
}
