' ********************************************************************************
'     Document    :  frmAudioCDBurn.vb
'     Description :  A form for burning audio disc's
' ********************************************************************************

Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports CDBurnerXP.Controls
Imports CDBurnerXP.Net
Imports Microsoft.Win32
Imports CDBurnerXP.Burning
Imports CDBurnerXP.Forms
Imports CDBurnerXP
Imports RocketDivision
Imports System.Runtime.InteropServices
Imports CDBurnerXP.Windows.Forms
Imports CDBurnerXP.IO

Public Class frmAudioCDBurn

  Private m_Device As StarBurnX.Drive
  Private m_AudioImage As AudioBurner
  Private m_TimeElapsed As String
  Private m_AudioCompilation As frmAudioCompilation = frmAudioCompilation.AudioInstance
  Private m_LayoutManager As AudioLayoutManager
  Private m_LabelTackAtOnce As String = "A pau&se for {0:0} seconds between all tracks"

#Region "Properties"

  Protected Overrides ReadOnly Property UseRemainingCalculator() As Boolean
    Get
      Return False
    End Get
  End Property

  Protected Overrides ReadOnly Property StarBurnDevice() As RocketDivision.StarBurnX.Drive
    Get
      Return m_Device
    End Get
  End Property

  Public Overrides Property NumberOfCopies() As Integer
    Get
      Return Convert.ToInt32(numCopies.Value)
    End Get
    Set(ByVal value As Integer)
      MyBase.NumberOfCopies = value
      numCopies.Value = CType(IIf(value > 0, value, 1), Integer)
    End Set
  End Property

#End Region

  <CLSCompliant(False)> _
  Public Sub New(ByRef mDevice As StarBurnX.Drive, ByRef mAudioImage As AudioBurner, ByVal layoutManager As AudioLayoutManager)

    ' This call is required by the Windows Form Designer.
    InitializeComponent()

    m_Device = mDevice
    m_AudioImage = mAudioImage
    m_LayoutManager = layoutManager

    AddHandler m_AudioImage.OnProgressExtended, AddressOf Me.HandleProgress
    AddHandler m_AudioImage.OnCloseSessionBegin, AddressOf Me.HandleCloseSession
    AddHandler m_AudioImage.OnTrackWriteBegin, AddressOf Me.HandleTrackWriteBegin
    AddHandler m_AudioImage.OnTestWriteDisabled, AddressOf Me.HandleTestWriteDisabled
  End Sub

  Private Sub frmAudioCDBurn_Load(ByVal sender As Object, _
                                  ByVal e As System.EventArgs) _
          Handles MyBase.Load
    'Language
    Me.Text = RM.GetString("Burn Audio CD")
    tabBurnOptions.Text = RM.GetString("Burn Options")
    tabBurnProgress.Text = RM.GetString("Burn Progress")
    cdbBS.BurnSpeedLabel = RM.GetString("Burn speed:")
    chkTestBurn.Text = RM.GetString("Simulate burning process")
    sepBurnMethod.Text = RM.GetString("Burn Method")
    sepSettings.Text = RM.GetString("Burn Options")
    optTrackAtOnce.Text = String.Format(RM.GetString(m_LabelTackAtOnce), numPauseLength.Value)
    optDiscAtOnce.Text = RM.GetString("N&o pause between tracks")
    cmdStartBurn.Text = RM.GetString("Start Burn")
    cmdCancel.Text = RM.GetString("Cancel")
    lblTime.Text = RM.GetString("Time Elapsed (hh:mm:ss):")
    lblStatus.Text = RM.GetString("Status: Idle")
    chkCloseDisc.Text = RM.GetString("Finalize Disc")
    chkShutdown.Text = RM.GetString("Shutdown computer after completion")
    lblRemaining.Text = String.Format(RM.GetString("Time Remaining (hh:mm:ss): {0}"), "")
    chkEject.Text = RM.GetString("Eject Disc After Burn")
    lblCopies.Text = RM.GetString("&Number of copies:")
    lblPauseLength.Text = RM.GetString("Paus&e length:")
    numPauseLength.Left = lblPauseLength.Right + 3

    BurningEngine.EnableLog(ApplicationSettings.LoggingEnabled)

    Animation.ImageList = MainToolStrip.Animations
    Animation.ShowFrame(0)
    m_TimeElapsed = RM.GetString("Time Elapsed (hh:mm:ss):") & " {0}"

    If m_Device Is Nothing Then
      ActivateControls(False)
    Else
      ActivateControls(True)

      cdbBS.BurnSpeedDeviceLetter = m_Device.DriveInfo.Letter
    End If

    ' Set last used burn speed, if present
    cdbBS.SetLastWriteSpeed = ApplicationSettings.LastAudioBurnSpeed
    chkCloseDisc.Checked = ApplicationSettings.AudioCloseDisc
    chkEject.Checked = ApplicationSettings.AudioBurnEject
    chkCDTextEnabled.Checked = ApplicationSettings.CDTextEnabled

    If ApplicationSettings.AudioPauseLength >= numPauseLength.Minimum AndAlso ApplicationSettings.AudioPauseLength <= numPauseLength.Maximum Then
      numPauseLength.Value = ApplicationSettings.AudioPauseLength
    End If

    optTrackAtOnce.Checked = True
    optDiscAtOnce.Checked = ApplicationSettings.AudioBurnGapless

    Me.Activate()

  End Sub

  Private Sub ApplyAudioWriterSettings()
    If optTrackAtOnce.Checked Then
      m_AudioImage.Mode = StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_TAO
      m_AudioImage.AudioPauseLength = Convert.ToInt32(numPauseLength.Value)
    Else
      If (m_Device.DriveInfo.SupportedModesWrite And StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_DAO_96) > 0 Then
        m_AudioImage.Mode = StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_DAO_96
      Else
        m_AudioImage.Mode = StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_DAO_16
      End If
    End If

    m_AudioImage.NextSessionAllowed = Not chkCloseDisc.Checked
    m_AudioImage.KeepSessionOpened = False 'Not chkCloseDisc.Checked
  End Sub

  Protected Overrides Sub ActivateControls(ByVal enable As Boolean)
    MyBase.ActivateControls(enable)
    cdbBS.Enabled = enable
    chkTestBurn.Enabled = enable AndAlso m_Device.DriveInfo.SupportTestWrite
    chkCloseDisc.Enabled = enable
    optDiscAtOnce.Enabled = enable
    optTrackAtOnce.Enabled = enable
    cmdStartBurn.Enabled = enable
  End Sub

  Private Sub StartBurn()

    ' Check if TAO or DAO is actually possible
    If optTrackAtOnce.Checked AndAlso Not ((m_Device.DriveInfo.SupportedModesWrite And StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_TAO) > 0) Then
      MessageBox.Show(Me, RM.GetString("Your burning device does not support the Track-at-Once mode (pauses between tracks)." + Environment.NewLine + "Choose the Disc-at-Once mode (without pauses between tracks) instead."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      Return
    End If

    If optDiscAtOnce.Checked AndAlso Not ((m_Device.DriveInfo.SupportedModesWrite And StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_DAO_16) > 0) AndAlso Not ((m_Device.DriveInfo.SupportedModesWrite And StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_DAO_96) > 0) Then
      MessageBox.Show(Me, RM.GetString("Your burning device does not support the Disc-at-Once mode (without pauses between tracks)." + Environment.NewLine + "Choose the Track-at-Once mode (pauses between tracks) instead."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      Return
    End If

    ' Warn CD-Text /TAO
    If optTrackAtOnce.Checked AndAlso m_LayoutManager.HasUserCDTextInformation Then
      If MessageBox.Show(Me, RM.GetString("You have entered CD-Text information. However, you chose to add pauses between tracks. CD-Text information can only be burned to disc, if you choose not to have any pauses." & Environment.NewLine & "Do you want to burn the disc anyway (CD-Text information will be lost)?"), RM.GetString("Lose CD-Text Information"), MessageBoxButtons.YesNo, MessageBoxIcon.Warning) <> System.Windows.Forms.DialogResult.Yes Then
        Exit Sub
      End If
    End If

    ' Set now, so that pause length can be validated
    ApplyAudioWriterSettings()

    ' Add tracks to image
    AudioConverter.ProcessReplayGain = CType(ApplicationSettings.AudioReplayGain, ReplayGainOption)

    RemoveHandler m_AudioImage.OnProgressExtended, AddressOf Me.HandleProgress
    Dim frmAddingTracks As New CDBurnerXP.Forms.AddingAudioTracks(m_LayoutManager, m_AudioImage, PathEx.GetTempRipPath())
    If frmAddingTracks.ShowDialog(Me) = DialogResult.Cancel Then
      Exit Sub
    End If

    ' Disable CD-Text if necessary
    If optDiscAtOnce.Checked AndAlso Not chkCDTextEnabled.Checked Then
      m_AudioImage.Artist = String.Empty
      m_AudioImage.Title = String.Empty

      For i As Integer = 0 To m_AudioImage.Tracks.Count - 1
        m_AudioImage.Tracks(i).Artist = String.Empty
        m_AudioImage.Tracks(i).Title = String.Empty
      Next
    End If

    AddHandler m_AudioImage.OnProgressExtended, AddressOf Me.HandleProgress

    If Not CheckDisc.Show(Me, m_Device, CheckDisc.DiscFeature.Writable, m_AudioImage, False, False) Then
      ResetUiAfterWork() ' escape from multiple copies
      Exit Sub
    End If

    ' Make sure that new speeds are loaded, after CheckDisc ensured that disc has been inserted
    cdbBS.UpdateBurningSpeeds()
    m_AudioImage.WriteSpeed = cdbBS.WriteSpeedLongValue

    'All Checks through can start burn here
    m_Status = Status.Burning
    m_StartTime = Now
    tmrElapsed.Enabled = True
    PrepareUiForWork()

    m_AudioImage.Drive = m_Device

    Dim burningThread As New Thread(New ParameterizedThreadStart(AddressOf StartBurnThread))
    burningThread.Start(chkTestBurn.Checked)
  End Sub

  Private Sub StartBurnThread(ByVal testBurn As Object)
    Try
      HandleBurnProcessStarted()

      m_AudioImage.Burn(CBool(testBurn))
    Catch ex As COMException
      Me.Invoke(New ParameterizedThreadStart(AddressOf HandleError), ex)
    Finally
      BurningEngine.TryUnlock(m_AudioImage.Drive)

      Me.BeginInvoke(New MethodInvoker(AddressOf StopWrite))
    End Try
  End Sub

  Sub HandleTrackWriteBegin(ByVal trackNumber As Integer)
    lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Burning track {0} of {1}", Culture), trackNumber, m_AudioImage.Tracks.Count))
    lblStatus.Text = String.Format(RM.GetString("Status: Burning track {0} of {1}", Culture), trackNumber, m_AudioImage.Tracks.Count)
  End Sub

  Protected Overrides Sub HandleError(ByVal ex As Object)
    MyBase.HandleError(ex, ApplicationSettings.LoggingEnabled, chkShutdown.Checked, m_AudioImage.LastErrorCode)
  End Sub

  Private Sub StopWrite()
    If m_BurningProcessCancelled Then
      If Animation.FirstFrame <> 74 Then '+1
        Animation.FirstFrame = 74 '+1
        Animation.LastFrame = 82 '+1
        Animation.Delay = 350
        Animation.Pause = 500
        Animation.StartAnimation()
      End If

      lblStatus.Text = RM.GetString("Writing cancelled...")
      lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Writing was cancelled by user"))

      'Completed Message
      TaskDialog.ShowOptionalMessageBox(Me, "AudioCancelled", Application.ProductName, "", RM.GetString("Writing was cancelled by user"), eTaskDialogButtons.OK, eSysIcons.Warning, 0)
    ElseIf m_AudioImage.LastErrorCode = StarBurnX.STARBURN_RESULT_CODES.STARBURN_OPERATION_SUCCEEDED Then
      lblStatus.Text = RM.GetString("Status: Idle")
      prgProgress.Value = 100
      Me.Text = String.Format(RM.GetString("{0}% Burning Audio CD"), prgProgress.Value)

      m_Status = Status.Idle
      tmrElapsed.Enabled = False

      If chkEject.Checked Then
        TryEject(m_Device)
      End If

      If chkShutdown.Checked Then
        ExitWindows.ForceShutdown()
      End If

      If Animation.FirstFrame <> 50 Then '+1
        Animation.FirstFrame = 50 '+1
        Animation.LastFrame = 61 '+1
        Animation.Delay = 90
        Animation.Pause = 500
        Animation.StartAnimation()
      End If

      Dim deltaTime As TimeSpan = Utility.TimeSpanSeconds(m_ElapsedTime - m_StartTime)
      Dim furtherCopies As Boolean = FurtherCopiesRequired()

      lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Burning process successfully completed."))
      If Not furtherCopies Then
        Dim msg As String = String.Format(RM.GetString( _
              "Burning process is successfully completed at speed {0}"), cdbBS.WriteSpeedDescription) _
              & Environment.NewLine & String.Format(RM.GetString( _
              "Total time for operation (hh:mm:ss): {0}"), deltaTime.ToString())
        TaskDialog.ShowOptionalMessageBox(Me, "AudioCompleted", Application.ProductName, "", msg, eTaskDialogButtons.OK, eSysIcons.Information, 0)
      End If

      If furtherCopies Then
        BeginBurnNextDisc()
        Exit Sub
      End If
    End If

    ResetUiAfterWork()
    lblRemaining.Text = String.Format(RM.GetString("Time Remaining (hh:mm:ss): {0}"), "")
    prgProgress.Value = 0
    Me.Text = RM.GetString("Burn Audio CD")
    m_OldTrackNum = 0
  End Sub

  Protected Overrides Sub OnStartBurnClick(ByVal sender As Object, ByVal e As System.EventArgs)
    MyBase.OnStartBurnClick(sender, e)
    StartBurn()
  End Sub

  Protected Overrides Sub OnCancelClick(ByVal sender As Object, ByVal e As System.EventArgs)
    MyBase.OnCancelClick(sender, e)

    If m_Status = Status.Burning Then
      If MessageBox.Show(Me, RM.GetString("Are you sure that you want to cancel the current burning process?"), Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) = DialogResult.Yes Then
        lblStatus.Text = RM.GetString("Aborting write, please wait...")
        lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("The writing was cancelled by the user, aborting please wait.."))

        Dim cancelThread As New Thread(New ThreadStart(AddressOf CancelBurningProcess))
        cancelThread.Start()
      End If
    Else
      Me.Close()
    End If
  End Sub

  Private Sub CancelBurningProcess()
    m_AudioImage.Cancel()
    m_BurningProcessCancelled = True
  End Sub

  Private Sub HandleProgress(ByVal tracknumber As Integer, ByVal isNextTrack As Boolean, ByVal totalPercent As Integer, ByVal timeRemaining As TimeSpan)
    SetTimeRemaining(timeRemaining)

    If m_BurningProcessCancelled Then
      lblStatus.Text = RM.GetString("Aborting write, please wait...", Culture)
    Else

      If Animation.FirstFrame <> 1 Then '+1
        Animation.FirstFrame = 1 '+1
        Animation.LastFrame = 18 '+1
        Animation.Delay = 80
        Animation.Pause = 50
        Animation.StartAnimation()

        ' Do not overwrite Track X status
        If lblStatus.Text = RM.GetString("Preparing to write disc...", Culture) Then
          lblStatus.Text = RM.GetString("Status: Burning disc...", Culture)
        End If
      End If

      prgProgress.Value = totalPercent
      Me.Text = String.Format(RM.GetString("{0}% Burning Audio CD", Culture), prgProgress.Value)
    End If
  End Sub

  Protected Overrides Sub OnClosed(ByVal e As System.EventArgs)
    ApplicationSettings.LastAudioBurnSpeed = cdbBS.WriteSpeedLongValue
    ApplicationSettings.AudioBurnEject = chkEject.Checked
    ApplicationSettings.AudioBurnGapless = optDiscAtOnce.Checked
    ApplicationSettings.AudioCloseDisc = chkCloseDisc.Checked
    ApplicationSettings.AudioPauseLength = Convert.ToInt32(numPauseLength.Value)
    ApplicationSettings.CDTextEnabled = chkCDTextEnabled.Checked

    RemoveHandler m_AudioImage.OnProgressExtended, AddressOf Me.HandleProgress
    RemoveHandler m_AudioImage.OnCloseSessionBegin, AddressOf Me.HandleCloseSession
    RemoveHandler m_AudioImage.OnTrackWriteBegin, AddressOf Me.HandleTrackWriteBegin
    RemoveHandler m_AudioImage.OnTestWriteDisabled, AddressOf Me.HandleTestWriteDisabled

    MyBase.OnClosed(e)
    Dispose(True)
  End Sub

  Private Sub numPauseLength_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numPauseLength.ValueChanged
    optTrackAtOnce.Text = String.Format(RM.GetString(m_LabelTackAtOnce), numPauseLength.Value)
    If optTrackAtOnce.Enabled Then
      optTrackAtOnce.Checked = True
    End If
  End Sub

  Private Sub optTrackAtOnce_EnabledChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optTrackAtOnce.EnabledChanged
    lblPauseLength.Enabled = optTrackAtOnce.Enabled
    numPauseLength.Enabled = optTrackAtOnce.Enabled
  End Sub

  Private Sub chkCDTextEnabled_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkCDTextEnabled.CheckedChanged
    If chkCDTextEnabled.Checked Then
      optDiscAtOnce.Checked = True
    End If
  End Sub

  Private Sub optDiscAtOnce_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optDiscAtOnce.CheckedChanged
    chkCDTextEnabled.Enabled = optDiscAtOnce.Checked AndAlso optDiscAtOnce.Enabled
  End Sub

  Private Sub optTrackAtOnce_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optTrackAtOnce.CheckedChanged
    chkCDTextEnabled.Enabled = optDiscAtOnce.Checked AndAlso optDiscAtOnce.Enabled
  End Sub
End Class
