' ********************************************************************************
'     Document    :  frmCDDVDBurn.vb
'     Description :  A form for burning the disc
' ********************************************************************************

Imports System.IO
Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports CDBurnerXP.Controls
Imports CDBurnerXP.Net
Imports CDBurnerXP.Burning
Imports Microsoft.Win32
Imports CDBurnerXP.Forms
Imports CDBurnerXP
Imports CDBurnerXP.Windows.Forms

Public Class frmCDDVDBurn

  Private m_Device As NMSDVDXLib.DeviceX
  Private m_DataImage As NMSDVDXLib.IDataImageX
  Private m_MultipleImages As NMSDVDXLib.IDataImageX()
  Private m_CurrentDataImageIndex As Integer = 0
  Private m_BurnSpeedText As String
  Private m_LayoutManager As FileLayoutManager
  Private m_VerifyErrors As Boolean = False
  Private m_CancelVerify As Boolean = False
  Private m_Caching As Boolean = False
  Private m_ErrorWhileCaching As Boolean = False
  Private m_BurnMsgSet As Boolean = False
  Private m_IsCachingDone As Boolean = False
  Private m_IsBurning As Boolean = False
  Private m_FormVerify As New frmVerifyFiles
  Private m_QuickBurnOption As DataQuickBurnOption
  Private m_CachingToDisc As String
  Private m_VerifyStatus As String
  Private m_VerifyCaption As String
  Private m_BurnStatus As String
  Private m_BurnCaption As String
  Private m_TimeElapsed As String = String.Empty

  Private Delegate Sub HandleVerifyCompleteDelegate(ByVal lImageID As Integer, ByVal lFileCount As Integer, ByVal lMatchedFiles As Integer, ByVal fileCountValid As Boolean, ByVal expectedFileCount As Integer, ByVal actualFileCount As Integer)
  Private Delegate Sub SetLabelDelegate(ByVal control As Label, ByVal text As String)

  Public Event WriteComplete()

#Region "Properties"

  Protected Overrides ReadOnly Property Device() As NMSDVDXLib.DeviceX
    Get
      Return m_Device
    End Get
  End Property

  Public Overrides Property NumberOfCopies() As Integer
    Get
      Return Convert.ToInt32(numCopies.Value)
    End Get
    Set(ByVal value As Integer)
      MyBase.NumberOfCopies = value
      numCopies.Value = Convert.ToInt32(IIf(value > 0, value, 1))
    End Set
  End Property

#End Region

  Protected Overrides Sub OnMediaChanged(ByVal VolumeLabel As String)
    ActivateControls(True)
  End Sub

  Public Sub New(ByVal layoutManager As FileLayoutManager, _
                 ByRef mDevice As NMSDVDXLib.DeviceX, _
                 Optional ByVal BurnOption As DataQuickBurnOption = DataQuickBurnOption.AdvancedSettings, Optional ByVal multipleDataImages As NMSDVDXLib.IDataImageX() = Nothing)

    ' This call is required by the Windows Form Designer.
    InitializeComponent()
    AcceptButton = cmdStartBurn
    CancelButton = cmdCancel

    ' Add any initialization after the InitializeComponent() call.
    m_LayoutManager = layoutManager
    m_Device = mDevice
    If Not multipleDataImages Is Nothing Then
      m_DataImage = multipleDataImages(0)
    Else
      m_DataImage = m_LayoutManager.DataImage
    End If
    m_QuickBurnOption = BurnOption
    m_MultipleImages = multipleDataImages

  End Sub

  Private Sub UpdateDiscSpanningInfo()
    lblDiscSpanning.Text = String.Format(RM.GetString("Disc spanning over {0} discs. Currently burning disc number:"), m_MultipleImages.Length)
    cboDiscNumber.SelectedItem = (m_CurrentDataImageIndex + 1)
    cboDiscNumber.Left = lblDiscSpanning.Right + 3
  End Sub

  Protected Function FurtherCopiesRequiredWithError(ByVal withError As Boolean) As Boolean
    Dim completedIndex As Integer = m_CurrentDataImageIndex

    Try
      Return Me.FurtherCopiesRequired()
    Finally
      If withError Then
        cboDiscNumber.ItemImages(completedIndex) = 2
      Else
        cboDiscNumber.ItemImages(completedIndex) = 1
      End If
    End Try
  End Function

  Protected Overrides Function FurtherCopiesRequired() As Boolean
    Dim furtherCopiesReq As Boolean = MyBase.FurtherCopiesRequired()
    If Not m_MultipleImages Is Nothing Then
      ' Are there further data images that need to be burned?
      Dim moreImagesAvailable As Boolean = (m_CurrentDataImageIndex < m_MultipleImages.Length - 1)
      If Not furtherCopiesReq Then
        If moreImagesAvailable Then
          ' Switch to next data image
          m_CopiesMade = 0
          m_CurrentDataImageIndex += 1
          m_DataImage = m_MultipleImages(m_CurrentDataImageIndex)
          UpdateDiscSpanningInfo()
          Return True
        Else
          ' End reached. Return false, but reset.
          m_CopiesMade = 0
          m_CurrentDataImageIndex = 0
          m_DataImage = m_MultipleImages(m_CurrentDataImageIndex)
          UpdateDiscSpanningInfo()
          Return False
        End If
        UpdateDiscSpanningInfo()
      End If
    End If

    Return furtherCopiesReq
  End Function

  Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
    MyBase.OnLoad(e)

    If DesignMode Then Exit Sub

    ' Adjust positioning for additional disc spanning info
    If Not m_MultipleImages Is Nothing Then
      picDiscSpanning.Visible = True
      lblDiscSpanning.Visible = True

      For i As Integer = 1 To m_MultipleImages.Length
        cboDiscNumber.Items.Add(i)
        cboDiscNumber.ItemImages(i - 1) = 0
      Next
      cboDiscNumber.SelectedIndex = 0
      cboDiscNumber.Visible = True
      Me.Height += picDiscSpanning.Height
      tabBurnTabs.Bounds = New Rectangle(tabBurnTabs.Left, tabBurnTabs.Top + picDiscSpanning.Height, tabBurnTabs.Width, tabBurnTabs.Height - picDiscSpanning.Height)
      UpdateDiscSpanningInfo()
    End If

    'Handlers for DVDEngine
    AddHandler WriteComplete, AddressOf Write_Complete
    'Language Set
    sepBurnMethod.Text = RM.GetString("Burn Method")
    sepSettings.Text = RM.GetString("Burn Options")
    optDAO.Text = RM.GetString("Disc at Once")
    optMS.Text = RM.GetString("Session at Once (add more files later)")
    m_BurnStatus = RM.GetString("Burning file {0} - {1}")
    m_BurnCaption = RM.GetString("{0}% Burning To Media - [{1}]")
    m_CachingToDisc = RM.GetString("{0}% Caching To Disc - [{1}]")
    m_VerifyStatus = RM.GetString("Status: Verifying file: {0}")
    m_VerifyCaption = RM.GetString("{0}% Verifying The Media - [{1}]")
    m_TimeElapsed = RM.GetString("Time Elapsed (hh:mm:ss): {0}")
    tabBurnOptions.Text = RM.GetString("Burn Options|Tab")
    tabBurnProgress.Text = RM.GetString("Burn Progress")
    cdbBS.BurnSpeedLabel = RM.GetString("Burn speed:")
    chkTestBurn.Text = RM.GetString("Simulate burning process")
    chkFinalize.Text = RM.GetString("Finalize Disc")
    chkMode2XA.Text = RM.GetString("Mode2XA instead of Mode1")
    chkVerify.Text = RM.GetString("Verify Data After Burn")
    chkBufferProtection.Text = RM.GetString("Use Buffer Protection")
    chkEject.Text = RM.GetString("Eject Disc After Burn")
    chkCache.Text = RM.GetString("Cache Compilation Before Burn")
    cmdStartBurn.Text = RM.GetString("Start Burn")
    cmdCancel.Text = RM.GetString("Cancel")
    cmdBootOptions.Text = RM.GetString("Set &boot options...")
    cmdLabels.Text = RM.GetString("Set Labels...")
    lblStatus.Text = RM.GetString("Status: Idle")
    lblTime.Text = String.Format(RM.GetString("Time Elapsed (hh:mm:ss): {0}"), "")
    lblRemaining.Text = String.Format(RM.GetString("Time Remaining (hh:mm:ss): {0}"), "")
    chkShutdown.Text = RM.GetString("Shutdown computer after completion")
    lblCopies.Text = RM.GetString("&Number of copies:")
    m_AutoImportEnabled = False
    Me.Text = String.Format(RM.GetString("Burn Disc - [{0}]"), m_LayoutManager.LayoutName)

    BurningEngine.EnableLog(ApplicationSettings.LoggingEnabled)

    cdbBS.BurnSpeedDeviceLetter = Device.GetDriveLetter()

    Animation.ImageList = MainToolStrip.Animations
    Animation.ShowFrame(0)

    'CheckBootable()

    chkEject.Checked = ApplicationSettings.DataBurnEject

    If Device.IsFeatureSupported(NMSDVDXLib.eFeatureX.dfBufferProtection) = True Then
      chkBufferProtection.Enabled = True
      chkBufferProtection.Checked = True

    Else
      chkBufferProtection.Enabled = False
      chkBufferProtection.Checked = False
    End If

    chkVerify.Checked = ApplicationSettings.DataBurnVerify
    chkFinalize.Checked = ApplicationSettings.DataBurnFinalize
    chkMode2XA.Checked = ApplicationSettings.DataMode2XA

    ' Set last used burn speed, if present
    cdbBS.SetLastWriteSpeed = ApplicationSettings.LastDataBurnSpeed

    ActivateControls(True)
    ' Remember setting
    If ApplicationSettings.DataUseDiscAtOnce And optDAO.Enabled Then
      optDAO.Checked = True
    End If

    If m_QuickBurnOption <> DataQuickBurnOption.AdvancedSettings Then
      chkTestBurn.Checked = False

      If m_QuickBurnOption = DataQuickBurnOption.LeaveOpen Then
        optMS.Checked = True
        chkFinalize.Checked = False
      Else
        If m_DataImage.IsImported() Then
          optMS.Checked = True
        Else
          optDAO.Checked = True
        End If

        chkFinalize.Checked = True
      End If

    End If

    Me.Activate()
  End Sub

  Protected Overrides Sub ActivateControls(ByVal enable As Boolean)
    MyBase.ActivateControls(enable)

    cdbBS.Enabled = enable
    cboDiscNumber.Enabled = enable

    ' All checkboxes..
    For Each chk In tabBurnOptions.Controls

      If TypeOf chk Is CheckBox Then
        CType(chk, CheckBox).Enabled = enable
      End If

    Next

    ' Can be used during burning process
    chkEject.Enabled = True
    chkVerify.Enabled = True
    chkShutdown.Enabled = True
    optDAO.Enabled = enable
    optMS.Enabled = enable

    cmdStartBurn.Enabled = enable

    cmdBootOptions.Enabled = enable
    cmdLabels.Enabled = enable

    If enable Then
      SetEnabledDiscType()
    End If

    If m_DataImage.IsImported() Then
      optMS.Checked = True
      optDAO.Enabled = False
    End If

  End Sub

  Public Sub SetEnabledDiscType()

    If Not Device Is Nothing AndAlso (Device.GetDiscInfo.IsCDMediaType Or (Device.GetDiscInfo.GetMediaType = NMSDVDXLib.eMediaTypeX.mtNOT_LOADED)) Then
      'Set CDR/CDRW Burning options
      chkCache.Enabled = True
      chkMode2XA.Enabled = True
      cdbBS.Enabled = True

    Else
      'Set DVD Burning options
      chkCache.Enabled = False
      chkCache.Checked = False
      chkFinalize.Enabled = True
      chkMode2XA.Enabled = False
      chkMode2XA.Checked = False
      cdbBS.Enabled = True
    End If

  End Sub

  Protected Overrides Sub OnStartBurnClick(ByVal sender As Object, ByVal e As System.EventArgs)
    MyBase.OnStartBurnClick(sender, e)

    If Not CheckDisc.Show(Me, Device, CheckDisc.DiscFeature.Writable, CheckDisc.DiscType.Any, Convert.ToInt64(m_DataImage.GetImageSizeBytes()), m_DataImage.IsImported(), False) Then
      ResetUiAfterWork() ' escape from multiple copies
      Exit Sub
    End If

    m_StartTime = Now
    tmrElapsed.Enabled = True

    'Set the properties of the write
    With m_DataImage
      .VerifyData = chkVerify.Checked
      .CloseDisc = chkFinalize.Checked 'Finalize

      .GetVolumeInfo.VolumeID = m_LayoutManager.LayoutName
      m_LayoutName = .GetVolumeInfo.VolumeID

      If chkCache.Checked Then
        .CacheFolder = Path.GetTempPath()
        .CacheMode = True
      Else
        .CacheMode = False
      End If

      ' Make sure that new speeds are loaded, after CheckDisc ensured that disc has been inserted
      cdbBS.UpdateBurningSpeeds()

      m_DataImage.WriteSpeed = cdbBS.WriteSpeedLongValue
      m_BurnSpeedText = cdbBS.WriteSpeedDescription
      RemainingCalculator.Init(Convert.ToInt64(m_DataImage.GetImageSizeBytes()), cdbBS.WriteSpeedLongValue * 1024)

      'Use Burn Proof/JustLink on this write?
      .BufferProtect = chkBufferProtection.Checked
      .DataWriteMode = CType(IIf(optDAO.Checked, NMSDVDXLib.eDataWriteModeX.dmDAO, NMSDVDXLib.eDataWriteModeX.dmMS), NMSDVDXLib.eDataWriteModeX)

      .TestMode = chkTestBurn.Checked

      If Device.GetDiscInfo.IsCDMediaType Then
        .Mode2XA = chkMode2XA.Checked

        If .TestMode = True Then
          .VerifyData = False
        End If

        If m_QuickBurnOption = DataQuickBurnOption.Finalise Then
          'burn and finalize
          .TestMode = False
          .CacheMode = False
          .CloseDisc = True

        ElseIf m_QuickBurnOption = DataQuickBurnOption.LeaveOpen Then
          'burn and leave open
          .TestMode = False
          .CacheMode = False
          .CloseDisc = False
        End If

      Else
        .DataWriteMode = CType(IIf(optDAO.Checked, NMSDVDXLib.eDataWriteModeX.dmDAO, NMSDVDXLib.eDataWriteModeX.dmMS), NMSDVDXLib.eDataWriteModeX)

        If m_QuickBurnOption = DataQuickBurnOption.Finalise Then
          'burn and finalize
          .CacheMode = False
          .CloseDisc = True

        ElseIf m_QuickBurnOption = DataQuickBurnOption.LeaveOpen Then
          .CacheMode = False
          .CloseDisc = False
        End If

      End If

    End With

    ' Now that all is set, make sure that user does not use 
    ' simulation if it's not applicable
    If m_DataImage.TestMode AndAlso (Not Device.IsFeatureSupported(NMSDVDXLib.eFeatureX.dfSimulateWrite) OrElse Not Device.GetDiscInfo().IsCDMediaType()) Then
      Dim cancel As Boolean
      HandleTestWriteDisabled(cancel)
      If (cancel) Then
        Exit Sub
      Else
        m_DataImage.TestMode = False
      End If
    End If

    PrepareUiForWork()

    m_IsBurning = True
    cmdCancel.Tag = "Burn"

    If m_DataImage.WriteDisc(Device) = False Then
      MessageBox.Show(Me, RM.GetString("Burn process could not be started."), _
              Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation, _
              MessageBoxDefaultButton.Button1)
      ResetUiAfterWork()
    End If

  End Sub
  Protected Overrides Sub EngineOnWriteStatus(ByVal lImageID As Integer, ByVal nImageProgress As Short, ByVal lImageBlocksWritten As Integer, ByVal lImageTotalBlocks As Integer, ByVal nCurrentTrack As Short, ByVal nTrackCount As Short, ByVal nTrackProgress As Short, ByVal lTrackBlocksWritten As Integer, ByVal lTrackTotalBlocks As Integer, ByVal status As NMSDVDXLib.eWriteStatusCodeX, ByRef bCancel As Boolean)
    Select Case status

      Case NMSDVDXLib.eWriteStatusCodeX.wsCachingImage
        'Debug.WriteLine("Caching Image")

        If nImageProgress > prgProgress.Value Then
          prgProgress.Value = nImageProgress

          If Animation.FirstFrame <> 105 And m_Caching = False And nImageProgress _
                  < 100 Then '+1
            m_Caching = True
            Animation.StopAnimation()
            Animation.FirstFrame = 105 '+1
            Animation.LastFrame = 123 '+1
            Animation.Delay = 90
            Animation.Pause = 200
            Animation.StartAnimation()
            lblStatus.Text = RM.GetString("Status: Creating cache image...")
            lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Caching of files started"))
            lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Creating cache image in the selected folder"))
          End If

          Me.Text = String.Format(m_CachingToDisc, nImageProgress, m_LayoutManager.LayoutName)

          If nImageProgress = 100 And m_Caching = True Then
            m_Caching = False
            prgProgress.Value = 0
            lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Caching of files completed"))

            If m_ErrorWhileCaching Then

              If MessageBox.Show(Me, RM.GetString( _
                      "One or more file access errors occured during the caching process. If you choose to burn still those files will not work on the burned disc. Do you want to contine with burning?"), _
                      Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Warning, _
                      MessageBoxDefaultButton.Button1) = DialogResult.No _
                      Then
                Exit Sub

              Else
                m_ErrorWhileCaching = False
              End If

            End If

            lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Burning disc from cache image at speed {0}"), m_BurnSpeedText))
            lblStatus.Text = RM.GetString("Status: Burning disc from cache image...")

            If Animation.FirstFrame <> 1 Then '+1
              Animation.FirstFrame = 1 '+1
              Animation.LastFrame = 18 '+1
              Animation.Delay = 80
              Animation.Pause = 50
              Animation.StartAnimation()
            End If

            m_IsCachingDone = True
          End If

        End If

      Case NMSDVDXLib.eWriteStatusCodeX.wsCachingTrack
        Debug.WriteLine("Caching Track")

      Case NMSDVDXLib.eWriteStatusCodeX.wsPreparingImage
        Debug.WriteLine("Preparing Image")

        If m_ErrorWhileCaching Then Exit Sub

        If Animation.FirstFrame <> 19 Then '+1
          Animation.FirstFrame = 19 '+ 1
          Animation.LastFrame = 34 '+1
          Animation.Delay = 80
          Animation.Pause = 200
          Animation.StartAnimation()

        Else
          Exit Sub
        End If

        lblStatus.Text = RM.GetString("Status: Preparing to burn disc...")
        lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Preparing to burn disc at {0}"), m_BurnSpeedText))
        prgProgress.Value = 0

      Case NMSDVDXLib.eWriteStatusCodeX.wsWritingLeadIn
        Debug.WriteLine("Writing Lead-In")
        lblStatus.Text = RM.GetString("Status: Writing Lead-In...")
        lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Writing Lead-In"))

      Case NMSDVDXLib.eWriteStatusCodeX.wsWritingLeadOut
        Debug.WriteLine("Writing Lead-Out")
        lblStatus.Text = RM.GetString("Status: Writing Lead-Out...")
        lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Writing Lead-Out"))

      Case NMSDVDXLib.eWriteStatusCodeX.wsWriting

      Case NMSDVDXLib.eWriteStatusCodeX.wsWriteCancelled
        Debug.WriteLine("Write Cancelled")
        tmrElapsed.Enabled = False

        If Animation.FirstFrame <> 74 Then '+1
          Animation.FirstFrame = 74 '+1
          Animation.LastFrame = 82 '+1
          Animation.Delay = 350
          Animation.Pause = 500
          Animation.StartAnimation()

        Else
          Exit Sub
        End If

        If m_ErrorWhileCaching Then
          m_ErrorWhileCaching = False
        End If

        lblStatus.Text = RM.GetString("Status: Burning cancelled...")
        lvwStatus.AddItem(LogListView.LogType.Warning, RM.GetString("Burning cancelled"))
        'Completed Message
        TaskDialog.ShowOptionalMessageBox(Me, "DataCancelled", Application.ProductName, "", RM.GetString("Burning was cancelled by the user"), eTaskDialogButtons.OK, eSysIcons.Warning, 0)
        lblStatus.Text = RM.GetString("Status: Idle")
        lblTime.Text = String.Format(RM.GetString( _
                "Time Elapsed (hh:mm:ss): {0}"), "")

        ResetUiAfterWork()
        m_BurnMsgSet = False
        prgProgress.Value = 0

      Case NMSDVDXLib.eWriteStatusCodeX.wsWriteError

      Case NMSDVDXLib.eWriteStatusCodeX.wsWriteSuccess
        Debug.WriteLine("Write Complete")
        Call Write_Complete()

      Case NMSDVDXLib.eWriteStatusCodeX.wsUpdatingVolume
        Debug.WriteLine("Updating Volume")

        If Animation.FirstFrame <> 35 Then '+1
          Animation.FirstFrame = 35 '+1
          Animation.LastFrame = 49 '+1
          Animation.Delay = 90
          Animation.Pause = 180
          Animation.StartAnimation()
        End If

        prgProgress.ShowMarquee = True

        lblStatus.Text = RM.GetString("Status: Closing track...")
        lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Closing track"))

      Case NMSDVDXLib.eWriteStatusCodeX.wsClosingTrack
        Debug.WriteLine("Closing Track")

        If Animation.FirstFrame <> 35 Then '+1
          Animation.FirstFrame = 35 '+1
          Animation.LastFrame = 49 '+1
          Animation.Delay = 90
          Animation.Pause = 180
          Animation.StartAnimation()
        End If

        prgProgress.ShowMarquee = True

        lblStatus.Text = RM.GetString("Status: Closing disc...")
        lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Closing disc"))

      Case NMSDVDXLib.eWriteStatusCodeX.wsVerifying

      Case Else
        'Do nothing

    End Select

    Dim intPercentTrackWritten As Integer

    If m_IsCachingDone Then
      prgProgress.Value = 0
      m_IsCachingDone = False
    End If

    If m_ErrorWhileCaching Then
      m_DataImage.CancelWrite()
      Exit Sub
    End If

    If Not (m_BurnMsgSet Or status = NMSDVDXLib.eWriteStatusCodeX.wsWriteSuccess) _
            Then

      If m_DataImage.TestMode Then
        lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Test burn process started at {0}"), m_BurnSpeedText))

      Else
        lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Burn process started at {0}"), m_BurnSpeedText))
      End If

      m_BurnMsgSet = True
    End If

    intPercentTrackWritten = Convert.ToInt32(Math.Min(100, ((lImageBlocksWritten / lImageTotalBlocks) * 100)))

    If intPercentTrackWritten > prgProgress.Value Then
      prgProgress.Value = intPercentTrackWritten
      Me.Text = String.Format(m_BurnCaption, intPercentTrackWritten, m_LayoutManager.LayoutName)
    End If

    Application.DoEvents()
  End Sub

  Protected Overrides Sub EngineOnWriteError(ByVal lImageID As Integer, ByRef objError As NMSDVDXLib.WriteErrorX)
    If chkShutdown.Checked Then
      ExitWindows.ForceShutdown()
    End If

    tmrElapsed.Enabled = False

    If Animation.FirstFrame <> 62 Then '+1
      Animation.FirstFrame = 62 '+1
      Animation.LastFrame = 73 '+1
      Animation.Delay = 90
      Animation.Pause = 500
      Animation.StartAnimation()

    Else
      Exit Sub
    End If

    prgProgress.ShowMarquee = False
    prgProgress.Value = 0

    Dim errorReport As New BurnErrorReport(objError)
    lvwStatus.AddItem(LogListView.LogType.Error, errorReport.Message)
    ' Set error image for current disc
    cboDiscNumber.ItemImages(m_CurrentDataImageIndex) = 2

    errorReport.ShowErrorAndAskForReport(Me, ApplicationSettings.LoggingEnabled)

    lblStatus.Text = RM.GetString("Status: Idle")
    lblTime.Text = String.Format(RM.GetString("Time Elapsed (hh:mm:ss): {0}"), "")
    ResetUiAfterWork()
    m_BurnMsgSet = False

    Try
      m_DataImage.CancelWrite()

    Catch ex As Exception

    End Try

    Device.Eject()
  End Sub

  Protected Overrides Sub EngineOnTrackFileWarning(ByVal lImageID As Integer, ByRef objError As NMSDVDXLib.FileErrorX, ByVal nTrackNumber As Integer, ByRef bIgnoreWarning As Boolean)
    Dim strErrorMsg As String = ""
    bIgnoreWarning = True
    strErrorMsg = String.Format(RM.GetString("File Warning: [{0} - {1}]"), _
            objError.GetFileName, objError.GetErrorMsg)
    'Show error message in the status bar...an error will also be raised as a write error
    lblStatus.Text = strErrorMsg
    lvwStatus.AddItem(LogListView.LogType.Error, strErrorMsg)

    If m_Caching Then
      m_ErrorWhileCaching = True
    End If

  End Sub

  Protected Overrides Sub EngineOnFileVerify(ByVal lImageID As Integer, ByRef objDataItem As NMSDVDXLib.DataItemX, ByVal dFileBytesCompared As Double, ByVal dFileBytesTotal As Double, ByVal fvStatus As NMSDVDXLib.eFileVerifyStatusCodeX, ByVal lFileIndex As Integer, ByVal lFileCount As Integer, ByRef bCancel As Boolean)
    Dim intPercentVerified As Integer

    If m_CancelVerify Then
      bCancel = True
      prgProgress.Value = 100
      Exit Sub
    End If

    Select Case fvStatus

      Case NMSDVDXLib.eFileVerifyStatusCodeX.fvsIncorrectByteCount 'Error - Incorrect byte count (source != disc)

        If Not m_VerifyErrors Then
          lvwStatus.AddItem(LogListView.LogType.Warning, RM.GetString("Verify Error (See detailed list on completion)"))
        End If

        m_VerifyErrors = True
        m_FormVerify.AddBusyFile(objDataItem.GetSourceFilePath, RM.GetString( _
                "Size of source and destination file did not match."))

      Case NMSDVDXLib.eFileVerifyStatusCodeX.fvsNoMatch 'Error - Bytes did not compare (source != disc)

        If Not m_VerifyErrors Then
          lvwStatus.AddItem(LogListView.LogType.Warning, RM.GetString("Verify Error (See detailed list on completion)"))
        End If

        m_VerifyErrors = True
        m_FormVerify.AddBusyFile(objDataItem.GetSourceFilePath, RM.GetString( _
                "Source and destination file did not match."))

      Case NMSDVDXLib.eFileVerifyStatusCodeX.fvsReadingDiscError 'Error - reading file from CD/DVD

        If Not m_VerifyErrors Then
          lvwStatus.AddItem(LogListView.LogType.Warning, RM.GetString("Verify Error (See detailed list on completion)"))
        End If

        m_VerifyErrors = True
        m_FormVerify.AddBusyFile(objDataItem.GetSourceFilePath, RM.GetString( _
                "Could not read destination file from disc."))

      Case NMSDVDXLib.eFileVerifyStatusCodeX.fvsReadingSourceError 'Error - reading source file

        If Not m_VerifyErrors Then
          lvwStatus.AddItem(LogListView.LogType.Warning, RM.GetString("Verify Error (See detailed list on completion)"))
        End If

        m_VerifyErrors = True
        m_FormVerify.AddBusyFile(objDataItem.GetSourceFilePath, RM.GetString( _
                "Could not read source file from disc."))

      Case Else
        'Do nothing

    End Select

    Dim shortPath As String = Utility.CompactString(objDataItem.GetImagePath, _
            lblStatus.Width, lblStatus.Font, m_VerifyStatus)
    lblStatus.Text = String.Format(m_VerifyStatus, shortPath)

    If dFileBytesTotal <= 0 Then
      intPercentVerified = Convert.ToInt32(((lFileIndex - 1) / lFileCount) * 100)
    Else
      intPercentVerified = Convert.ToInt32((((lFileIndex - 1) / lFileCount) + (dFileBytesCompared / dFileBytesTotal) / lFileCount) * 100)
    End If

    If intPercentVerified > prgProgress.Value Then
      Me.Text = String.Format(m_VerifyCaption, intPercentVerified, m_LayoutManager.LayoutName)
      prgProgress.Value = intPercentVerified
      Application.DoEvents()
    End If

  End Sub

  Private Sub SetLabel(ByVal control As Label, ByVal text As String)
    control.Text = RM.GetString(text)
  End Sub

  Private Sub AdditionalVerification(ByVal data As Object)
    Dim intValues As Integer() = CType(data, Integer())
    Dim lImageID As Integer = intValues(0)
    Dim lFileCount As Integer = intValues(1)
    Dim lMatchedFiles As Integer = intValues(2)

    Dim actualFileCount As Integer
    Dim verifyResult As Boolean = True

    If Not m_CancelVerify Then
      Me.Invoke(New SetLabelDelegate(AddressOf SetLabel), lblStatus, "Status: Verifying file count")

      Try
        actualFileCount = Directory.GetFiles(m_Device.GetDriveLetter() + ":\", "*", SearchOption.AllDirectories).Length
        verifyResult = (m_DataImage.GetFileLayout().GetFileCount() = actualFileCount)
      Catch
        ' An error occured, we don't know what happens. Do not assume a negative result.
      End Try
    End If

    Me.BeginInvoke(New HandleVerifyCompleteDelegate(AddressOf HandleVerifyComplete), lImageID, lFileCount, lMatchedFiles, verifyResult, m_DataImage.GetFileLayout().GetFileCount(), actualFileCount)
  End Sub

  Protected Overrides Sub EngineOnFileVerifyComplete(ByVal lImageID As Integer, ByVal lFileCount As Integer, ByVal lMatchedFiles As Integer)
    Dim verifyThread As New Thread(AddressOf AdditionalVerification)
    verifyThread.Start(New Integer() {lImageID, lFileCount, lMatchedFiles})
  End Sub

  Private Sub HandleVerifyComplete(ByVal lImageID As Integer, ByVal lFileCount As Integer, ByVal lMatchedFiles As Integer, ByVal fileCountValid As Boolean, ByVal expectedFileCount As Integer, ByVal actualFileCount As Integer)
    If Animation.FirstFrame <> 50 Then '+1
      Animation.FirstFrame = 50 '+1
      Animation.LastFrame = 61 '+1
      Animation.Delay = 90
      Animation.Pause = 500
      Animation.StartAnimation()
    Else
      Exit Sub
    End If

    tmrElapsed.Enabled = False
    tmrElapsed.Stop()
    lblStatus.Text = RM.GetString("Status: Idle")
    lblTime.Text = String.Format(RM.GetString("Time Elapsed (hh:mm:ss): {0}"), "")
    prgProgress.Style = ProgressBarStyle.Blocks
    prgProgress.Value = 100

    If chkEject.Checked Then
      Device.Eject()
    End If

    If m_CancelVerify Then
      lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Verifying was cancelled by user."))
      lblStatus.Text = RM.GetString("Status: Verify cancelled.")

    Else
      lblStatus.Text = String.Format(RM.GetString( _
              "Status: Verify completed ({0} of {1} files matched)."), lMatchedFiles, _
              lFileCount)
      lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Verify completed ({0} of {1} files matched)."), lMatchedFiles, lFileCount))
    End If

    If chkShutdown.Checked Then
      ExitWindows.ForceShutdown()
    End If

    If m_VerifyErrors Then
      m_VerifyErrors = False
      lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Burning process is completed at {0}"), m_BurnSpeedText))

      Dim deltatime As TimeSpan = Utility.TimeSpanSeconds(m_ElapsedTime - m_StartTime)

      If MessageBox.Show(Me, String.Format(RM.GetString( _
              "Burning process at speed {0} is now completed. One or more files did not pass the verification."), _
              m_BurnSpeedText) & Environment.NewLine & String.Format(RM.GetString( _
              "Total time for operation (hh:mm:ss): {0}"), deltatime.ToString()) & RM.GetString( _
              "Do you want to see a report of the files which failed verification?"), _
              Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Warning, _
              MessageBoxDefaultButton.Button1) = DialogResult.Yes Then
        m_FormVerify.ShowDialog(Me)

        If FurtherCopiesRequiredWithError(True) Then
          BeginBurnNextDisc()
          Exit Sub
        End If
      End If

    ElseIf m_CancelVerify Then
      m_CancelVerify = False
      lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Burning process is successfully completed at {0}"), m_BurnSpeedText))

      Dim deltatime As TimeSpan = Utility.TimeSpanSeconds(m_ElapsedTime - m_StartTime)
      Dim msg As String = String.Format(RM.GetString( _
              "Burning process completed at speed {0}, verifying of the disc was cancelled."), _
              m_BurnSpeedText) & Environment.NewLine & String.Format(RM.GetString( _
              "Total time for operation (hh:mm:ss): {0}"), deltatime.ToString())
      TaskDialog.ShowOptionalMessageBox(Me, "DataCompletedVerifyCancelled", Application.ProductName, "", msg, eTaskDialogButtons.OK, eSysIcons.Information, 0)

    Else
      lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Burning process is successfully completed at {0}"), m_BurnSpeedText))

      Dim deltatime As TimeSpan = Utility.TimeSpanSeconds(m_ElapsedTime - m_StartTime)

      Dim furtherCopies As Boolean = FurtherCopiesRequiredWithError(False)

      If Not fileCountValid Then
        MessageBox.Show(Me, String.Format(RM.GetString("The data has been written correctly to the disc, but the number of files does not match the expected file count. Most likely, the file system structure has not been written correctly and some files will not appear on the disc." & Environment.NewLine & Environment.NewLine & _
        "Expected file count: {0}" & Environment.NewLine & _
        "Actual file count: {1}"), expectedFileCount, actualFileCount), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning)
      ElseIf Not furtherCopies Then
        Dim msg As String = String.Format(RM.GetString( _
              "Burning at speed {0} and verifying of the disc successfully completed."), _
              m_BurnSpeedText) & Environment.NewLine & String.Format(RM.GetString( _
              "Total time for operation (hh:mm:ss): {0}"), deltatime.ToString())
        TaskDialog.ShowOptionalMessageBox(Me, "DataCompletedVerifyOK", Application.ProductName, "", msg, eTaskDialogButtons.OK, eSysIcons.Information, 0)
      End If

      If furtherCopies Then
        BeginBurnNextDisc()
        Exit Sub
      End If
    End If

    ResetUiAfterWork()

  End Sub

  Protected Overrides Sub EngineOnFileVerifyStart(ByVal lImageID As Integer, ByVal lFileCount As Integer)
    If Animation.FirstFrame <> 124 Then '+1
      Animation.FirstFrame = 124 '+1
      Animation.LastFrame = 143 '+1
      Animation.Delay = 90
      Animation.Pause = 180
      Animation.StartAnimation()

    Else
      Exit Sub
    End If

    prgProgress.ShowMarquee = False
    prgProgress.Value = 0

    cmdCancel.Tag = "Verify"
    lblStatus.Text = String.Format(RM.GetString( _
            "Status: Verifying of files started ({0} files to compare)"), lFileCount)
    lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Verifying of files started ({0} files to compare)"), lFileCount))
  End Sub

  Protected Overrides Sub EngineOnTrackFileStatus(ByVal lImageID As Integer, ByVal nTrackNumber As Integer, ByVal lFileCount As Integer, ByVal lFileIndex As Integer, ByVal sFileName As String)
    If Animation.FirstFrame <> 1 Then '+1
      Animation.FirstFrame = 1 '+1
      Animation.LastFrame = 18 '+1
      Animation.Delay = 80
      Animation.Pause = 50
      Animation.StartAnimation()
    End If

    If m_Caching = False Then

      Dim shortPath As String = Utility.CompactString(sFileName, lblStatus.Width, _
              lblStatus.Font, m_BurnStatus)
      lblStatus.Text = String.Format(m_BurnStatus, lFileIndex, shortPath)

    Else

      Dim cacheString As String = RM.GetString("Caching file: {0} - {1}")
      lblStatus.Text = String.Format(cacheString, lFileIndex, sFileName)
    End If

  End Sub

  Private Sub Write_Complete()

    lblStatus.Text = RM.GetString("Status: Burning completed...")
    prgProgress.Style = ProgressBarStyle.Blocks
    prgProgress.Value = 100
    Application.DoEvents()
    m_BurnMsgSet = False

    If m_DataImage.VerifyData = False Then

      tmrElapsed.Enabled = False

      If Animation.FirstFrame <> 50 Then '+1
        Animation.FirstFrame = 50 '+1
        Animation.LastFrame = 61 '+1
        Animation.Delay = 90
        Animation.Pause = 500
        Animation.StartAnimation()

      Else
        Exit Sub
      End If

      prgProgress.ShowMarquee = False

      lblStatus.Text = RM.GetString("Status: Idle")
      lblTime.Text = String.Format(RM.GetString("Time Elapsed (hh:mm:ss): {0}"), "")
      prgProgress.Style = ProgressBarStyle.Blocks
      prgProgress.Value = 100

      Dim bContinueBurnAfterTest As Boolean = False

      If m_DataImage.TestMode Then
        lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Test burning process is successfully completed at speed {0}"), m_BurnSpeedText))

        Dim deltaTime As TimeSpan = Utility.TimeSpanSeconds(m_ElapsedTime - m_StartTime)

        If MessageBox.Show(Me, String.Format(RM.GetString( _
                "Test burning process is successfully completed at speed {0}"), _
                m_BurnSpeedText) & Environment.NewLine & String.Format( _
                RM.GetString("Total time for operation (hh:mm:ss): {0}"), deltaTime.ToString()) & Environment.NewLine & _
                RM.GetString("Do you want to burn the disc now?"), Application.ProductName, _
                MessageBoxButtons.YesNo, MessageBoxIcon.Question, _
                MessageBoxDefaultButton.Button2) = DialogResult.Yes Then

          chkTestBurn.Checked = False
          bContinueBurnAfterTest = True
        End If

      Else
        If chkEject.Checked Then
          Device.Eject()

        Else
          Device.UnMount()
          Device.Mount()
        End If

        If chkShutdown.Checked Then
          ExitWindows.ForceShutdown()
        End If

        lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Burning process is successfully completed at speed {0}"), m_BurnSpeedText))

        Dim deltaTime As TimeSpan = Utility.TimeSpanSeconds(m_ElapsedTime - m_StartTime)

        Dim furtherCopies As Boolean = FurtherCopiesRequiredWithError(False)

        If Not furtherCopies Then
          Dim msg As String = String.Format(RM.GetString( _
                "Burning process is successfully completed at speed {0}"), _
                m_BurnSpeedText) & Environment.NewLine & String.Format( _
                RM.GetString("Total time for operation (hh:mm:ss): {0}"), deltaTime.ToString())
          TaskDialog.ShowOptionalMessageBox(Me, "DataCompleted", Application.ProductName, "", msg, eTaskDialogButtons.OK, eSysIcons.Information, 0)
        End If

        If furtherCopies Then
          BeginBurnNextDisc()
          Exit Sub
        End If
      End If

      ResetUiAfterWork()

      If bContinueBurnAfterTest Then
        bContinueBurnAfterTest = False
        tmrTestBurn.Start() 'Timer that will give the process some time before start burning again
        Exit Sub
      End If
    End If

  End Sub

  Private Sub frmCDDVDBurn_FormClosing(ByVal sender As Object, _
                                       ByVal e As System.Windows.Forms.FormClosingEventArgs) _
          Handles Me.FormClosing

    If m_IsBurning Then
      e.Cancel = True
      Exit Sub
    End If

    If m_QuickBurnOption = DataQuickBurnOption.AdvancedSettings Then
      ApplicationSettings.DataBurnFinalize = chkFinalize.Checked
      ApplicationSettings.DataUseDiscAtOnce = optDAO.Checked
    End If

    ApplicationSettings.DataBurnVerify = chkVerify.Checked
    ApplicationSettings.DataBurnEject = chkEject.Checked
    ApplicationSettings.DataMode2XA = chkMode2XA.Checked
    ApplicationSettings.LastDataBurnSpeed = cdbBS.WriteSpeedLongValue

    RemoveHandler WriteComplete, AddressOf Write_Complete
    m_AutoImportEnabled = True
  End Sub

  Protected Overrides Sub OnLoaded(ByVal sender As Object, ByVal e As System.EventArgs)
    If m_QuickBurnOption <> DataQuickBurnOption.AdvancedSettings Then
      cmdStartBurn.PerformClick()
    End If
  End Sub

  Private Sub cmdBootOptions_Click(ByVal sender As System.Object, _
                                   ByVal e As System.EventArgs) _
          Handles cmdBootOptions.Click

    Dim frmBoot As New frmCDDVDBootOptions(m_LayoutManager)
    frmBoot.ShowDialog(Me)
    Application.DoEvents()
  End Sub

  Private Sub cmdLabels_Click(ByVal sender As System.Object, _
                              ByVal e As System.EventArgs) _
          Handles cmdLabels.Click

    Dim frmLabels As New frmCDDVDLabel(m_LayoutManager)
    frmLabels.ShowDialog(Me)
    Application.DoEvents()
  End Sub

  Private Sub chkTestBurn_CheckedChanged(ByVal sender As Object, _
                                         ByVal e As System.EventArgs) _
          Handles chkTestBurn.CheckedChanged

    If chkVerify.Checked Then
      chkVerify.Checked = False
    End If

    chkVerify.Enabled = Not chkTestBurn.Checked
  End Sub

  Private Sub chkVerify_CheckedChanged(ByVal sender As System.Object, _
                                       ByVal e As System.EventArgs) _
          Handles chkVerify.CheckedChanged

    If chkVerify.Checked Then
      m_DataImage.VerifyData = True

    Else
      m_DataImage.VerifyData = False
    End If

  End Sub

  Private Sub optMS_CheckedChanged(ByVal sender As System.Object, _
                                   ByVal e As System.EventArgs) _
          Handles optMS.CheckedChanged
    chkFinalize.Enabled = True
  End Sub

  Private Sub optDAO_CheckedChanged(ByVal sender As System.Object, _
                                    ByVal e As System.EventArgs) _
          Handles optDAO.CheckedChanged
    chkFinalize.Checked = True
    chkFinalize.Enabled = False
  End Sub

  Protected Overrides Sub OnCancelClick(ByVal sender As Object, ByVal e As System.EventArgs)
    MyBase.OnCancelClick(sender, e)

    If cmdCancel.Tag Is Nothing Then
      ' need updated main window after burning? not sure about this yet.
      ' tmpDevice.Refresh(False)
      ' Application.DoEvents()
      Me.Close()
      Exit Sub
    End If

    Select Case cmdCancel.Tag.ToString

      Case "Burn"

        If MessageBox.Show(Me, RM.GetString( _
                "Are you sure that you want to cancel the current burning process?"), _
                Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question, _
                MessageBoxDefaultButton.Button2) = DialogResult.Yes Then
          cmdCancel.Tag = Nothing
          lblStatus.Text = RM.GetString("Status: Aborting burn, please wait...")
          lvwStatus.AddItem(LogListView.LogType.Warning, RM.GetString("Burning was cancelled by the user, aborting please wait"))
          m_DataImage.CancelWrite()

        Else
          Exit Sub
        End If

      Case "Verify"

        If MessageBox.Show(Me, RM.GetString( _
                "Are you sure that you want to cancel the verify process?"), _
                Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question, _
                MessageBoxDefaultButton.Button2) = DialogResult.Yes Then
          cmdCancel.Tag = Nothing
          m_CancelVerify = True

        Else
          Exit Sub
        End If

      Case Else
        'Do nothing

    End Select
  End Sub

  Protected Overrides Sub ResetUiAfterWork()
    MyBase.ResetUiAfterWork()
    m_IsBurning = False
    Me.Text = String.Format(RM.GetString("Burn Disc - [{0}]"), m_LayoutManager.LayoutName)
  End Sub

  Private Sub tmrTestBurn_Tick(ByVal sender As System.Object, _
                               ByVal e As System.EventArgs) _
          Handles tmrTestBurn.Tick
    tmrTestBurn.Enabled = False
    cmdStartBurn.PerformClick()
  End Sub

  Private Sub cboDiscNumber_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboDiscNumber.SelectedIndexChanged
    m_CurrentDataImageIndex = cboDiscNumber.SelectedIndex
    m_DataImage = m_MultipleImages(m_CurrentDataImageIndex)
  End Sub

  Protected Overrides Sub OnClosed(ByVal e As System.EventArgs)
    MyBase.OnClosed(e)
    Dispose(True)
  End Sub

End Class
