'*************************************************************************
' Klass: frmCDDVDConvertISO
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-24 18:14
' Kommentarer: 
'*************************************************************************

Imports System.IO
Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports CDBurnerXP.Controls
Imports CDBurnerXP.Net
Imports CDBurnerXP.IO
Imports CDBurnerXP.Burning
Imports CDBurnerXP.Forms
Imports CDBurnerXP
Imports RocketDivision.StarBurnX
Imports System.Runtime.InteropServices

Public Class frmCDDVDCopy
  Private grabber As New CDBurnerXP.Burning.Grabber()
  Private imageBurner As New CDBurnerXP.Burning.ImageBurner()
  Private targetImageType As Integer = 0
  Private imageFileName As String
  Private burnAfterGrabbing As Boolean = False
  Private cachedGrabInfo As Dictionary(Of Int32, CDBurnerXP.Burning.Grabber.GrabIsoInfo) = Nothing
  Private Delegate Function PrepareBurnDelegate(ByVal grabDiscInfo As Dictionary(Of Int32, CDBurnerXP.Burning.Grabber.GrabIsoInfo)) As Boolean

#Region "Properties"

  Public Overrides Property NumberOfCopies() As Integer
    Get
      Return Convert.ToInt32(numCopies.Value)
    End Get
    Set(ByVal value As Integer)
      MyBase.NumberOfCopies = value
      numCopies.Value = Convert.ToInt32(IIf(value > 0, value, 1))
    End Set
  End Property

  Protected Overrides ReadOnly Property UseRemainingCalculator() As Boolean
    Get
      Return False
    End Get
  End Property

  Protected Overrides ReadOnly Property StarBurnDevice() As RocketDivision.StarBurnX.Drive
    Get
      Return Me.imageBurner.Drive
    End Get
  End Property

#End Region

  Public Sub New()
    ' This call is required by the Windows Form Designer.
    InitializeComponent()
    cbDevice_CheckedChanged(Me, EventArgs.Empty)
    cboImageType.SelectedIndex = 0

    AddHandler grabber.OnProgressExtended, AddressOf OnGrabberProgress
    AddHandler grabber.OnBadBlockHit, AddressOf OnGabberBadBlockHit
    grabber.OnBadBlockHitEnabled = True
    AddHandler imageBurner.OnProgressExtended, AddressOf OnImageBurnerProgress
  End Sub

  Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
    MyBase.OnLoad(e)

    If DesignMode Then Exit Sub

    'Language Set
    Me.Text = RM.GetString("Copy Disc.TitleBar")
    tabBurnOptions.Text = RM.GetString("Copy Options")
    tabBurnProgress.Text = RM.GetString("Copy Progress")
    sepTarget.Text = RM.GetString("Target")
    sepSource.Text = RM.GetString("Source")
    lblSourceDevice.Text = RM.GetString("&Device:")
    lblReadRetries.Text = RM.GetString("&Read retries:")
    chkIgnoreUnreadable.Text = RM.GetString("Ignore &unreadable data")
    chkDisableHardwareEC.Text = RM.GetString("&Disable hardware error correction")
    cbDevice.Text = RM.GetString("D&evice") & ":"
    chkFinalize.Text = RM.GetString("&Finalize disc")
    cbHardDisc.Text = RM.GetString("&Hard disc")
    cmdStartBurn.Text = RM.GetString("Copy disc")
    cmdCancel.Text = RM.GetString("Cancel")
    lblStatus.Text = RM.GetString("Status: Idle")
    lblImagePath.Text = RM.GetString("File &name:")
    lblImageType.Text = RM.GetString("&File type:")
    lblTime.Text = String.Format(RM.GetString("Time Elapsed (hh:mm:ss): {0}"), "")
    lblRemaining.Text = String.Format(RM.GetString("Time Remaining (hh:mm:ss): {0}"), "")
    chkEject.Text = RM.GetString("Eject Disc After Burn")
    lblCopies.Text = RM.GetString("&Number of copies:")

    ' Adjust dialog
    cbTargetDevice.Left = cbDevice.Right
    cbTargetDevice.Width = tabBurnOptions.Width - cbDevice.Width - 10

    ' Align second checkbox to numeric up/down
    If chkFinalize.Width < lblCopies.Width Then
      chkFinalize.AutoSize = False
      chkFinalize.Width = lblCopies.Width - 1
    End If

    cboImageType.Items.Clear()
    cboImageType.Items.Add(RM.GetString("MDS (Whole disc)"))
    cboImageType.Items.Add(RM.GetString("ISO (Single track)"))
    cboImageType.SelectedIndex = 0

    BurningEngine.EnableLog(ApplicationSettings.LoggingEnabled)

    m_AutoImportEnabled = False

    If Not cbTargetDevice.DeviceSelect.HasDrives Then
      cbHardDisc.Checked = True
    End If
    ActivateControls(True)

    chkDisableHardwareEC.Checked = ApplicationSettings.CopyDisableHardwareEC
    chkEject.Checked = ApplicationSettings.CopyEjectDisc
    chkFinalize.Checked = ApplicationSettings.CopyFinaliseDisc
    numReadRetries.Value = ApplicationSettings.CopyReadRetries
    chkIgnoreUnreadable.Checked = ApplicationSettings.CopyIgnoreUnreadableData
    cboImageType.SelectedIndex = ApplicationSettings.CopyImageType
    cbSourceDevice.SelectedDeviceLetter = ApplicationSettings.CopySourceDevice

    Animation.ImageList = MainToolStrip.Animations
    Animation.ShowFrame(0)
  End Sub

  Protected Overrides Sub ActivateControls(ByVal enable As Boolean)
    MyBase.ActivateControls(enable)

    chkFinalize.Enabled = enable AndAlso cbDevice.Checked
    chkDisableHardwareEC.Enabled = enable
    chkIgnoreUnreadable.Enabled = enable
    numReadRetries.Enabled = enable
    cbSourceDevice.Enabled = enable
    lblSourceDevice.Enabled = enable
    lblReadRetries.Enabled = enable
    cbTargetDevice.Enabled = enable AndAlso cbDevice.Checked
    chkEject.Enabled = cbDevice.Checked

    cbDevice.Enabled = enable AndAlso cbTargetDevice.DeviceSelect.HasDrives

    cbHardDisc.Enabled = enable
    lblImageType.Enabled = enable AndAlso cbHardDisc.Checked
    cboImageType.Enabled = enable AndAlso cbHardDisc.Checked
    lblImagePath.Enabled = enable AndAlso cbHardDisc.Checked
    txtImagePath.Enabled = enable AndAlso cbHardDisc.Checked
    cmdSelectPath.Enabled = enable AndAlso cbHardDisc.Checked
  End Sub

  Protected Overrides Sub ResetUiAfterWork()
    Me.cachedGrabInfo = Nothing
    MyBase.ResetUiAfterWork()
  End Sub

  Private Sub OnGabberBadBlockHit(ByVal StartLBA As Integer, ByVal CountLBA As Integer)
    Dim diffToEnd As Integer = grabber.Drive.DiscInfo.TotalSize - StartLBA
    If diffToEnd >= 150 Then
      lvwStatus.AddItem(LogListView.LogType.Warning, String.Format(RM.GetString("Unreadable area detected on disc at position {0}"), StartLBA))
    End If
  End Sub

  Protected Sub OnGrabberProgress(ByVal tracknumber As Integer, ByVal isNextTrack As Boolean, ByVal totalPercent As Integer, ByVal timeRemaining As TimeSpan)
    SetTimeRemaining(timeRemaining)

    If cbHardDisc.Checked Then
      prgProgress.Value = CInt(totalPercent)
    Else
      ' We still have a second part here
      prgProgress.Value = CInt(totalPercent / 2)
    End If
    Me.Text = String.Format(RM.GetString("{0}% Copying Disc"), prgProgress.Value)
  End Sub

  Private Sub OnImageBurnerProgress(ByVal percent As Integer, ByVal timeRemaining As Integer)
    If Animation.FirstFrame <> 1 Then '+1
      Animation.FirstFrame = 1 '+1
      Animation.LastFrame = 18 '+1
      Animation.Delay = 80
      Animation.Pause = 50
      Animation.StartAnimation()

      lblStatus.Text = RM.GetString("Writing disc...")
      lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Writing process started at {0}"), cbTargetDevice.WriteSpeedDescription))
    End If

    SetTimeRemaining(TimeSpan.FromSeconds(timeRemaining))

    prgProgress.Value = CInt(50 + percent / 2)

    Me.Text = String.Format(RM.GetString("{0}% Copying Disc"), prgProgress.Value)
  End Sub

  Protected Overrides Sub OnStartBurnClick(ByVal sender As Object, ByVal e As System.EventArgs)
    MyBase.OnStartBurnClick(sender, e)

    If cbHardDisc.Checked Then
      'We will save image to hdd, check so a valid path has been entered
      If txtImagePath.Text = "" Then
        MessageBox.Show(Me, RM.GetString("You did not choose a target file name for the image file. Select a file name to proceed with copying the disc."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.None)
        Exit Sub
      End If
    End If

    ' Check if source disc is present
    If Me.cachedGrabInfo Is Nothing Then
      Try
        cbSourceDevice.SelectedStarBurnDevice.DiscInfo.Refresh()
        If cbSourceDevice.SelectedStarBurnDevice.DiscInfo.DiscType = STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_NONE Then
          MessageBox.Show(Me, RM.GetString("The disc cannot be copied, because there is no disc present in the source device. Please insert the disc you want to copy."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
          Exit Sub
        End If
      Catch ex As COMException
        If Not HandleStarBurnError(Me, ex, cbSourceDevice.SelectedStarBurnDevice) Then
          Throw ex
        End If
        Exit Sub
      End Try
    End If

    ' Determine image target path
    If cbDevice.Checked Then
      ' Create folder for multiple tracks
      Me.imageFileName = Path.Combine(PathEx.GetTempPath(), Guid.NewGuid().ToString())
    Else
      Me.imageFileName = txtImagePath.Text
    End If

    ' Either check temp location or target path for required space
    If Not DiskSpaceRequiredDialog.Show(Me, cbSourceDevice.SelectedStarBurnDevice.DiscInfo.GetTotalUsedSize(), Me.imageFileName) Then
      Exit Sub
    End If

    ' Check if source contains multiple tracks. If so, suggest
    ' MDS (which can hold multiple tracks).
    If cboImageType.Enabled AndAlso cboImageType.SelectedIndex <> 0 Then
      Dim trackCount As Integer = 0
      For i As Integer = 0 To cbSourceDevice.SelectedStarBurnDevice.DiscInfo.Sessions.Count - 1
        Dim session As Session = cbSourceDevice.SelectedStarBurnDevice.DiscInfo.Sessions(i)

        For j As Integer = 0 To session.Tracks.Count - 1
          trackCount += 1
        Next
      Next
      If trackCount > 1 Then
        If MessageBox.Show(Me, String.Format(RM.GetString("The disc you would like to copy contains more than one track ({0}). When using ISO (instead of MDS) as target image type, only the first track will be saved." & Environment.NewLine & Environment.NewLine & "Do you want to use ISO anyway?"), trackCount), Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) = System.Windows.Forms.DialogResult.No Then
          Exit Sub
        End If
      End If
    End If

    grabber.Drive = cbSourceDevice.SelectedStarBurnDevice
    grabber.IgnoreBadBlocks = chkIgnoreUnreadable.Checked
    grabber.DisableHardwareErrorCorrection = chkDisableHardwareEC.Checked
    grabber.NumberOfReadRetries = CInt(numReadRetries.Value)
    Me.targetImageType = If(cboImageType.SelectedIndex = 0 OrElse cbDevice.Checked, 0, 1)
    Me.burnAfterGrabbing = cbDevice.Checked

    PrepareUiForWork()
    m_StartTime = Now
    tmrElapsed.Enabled = True
    m_Status = Status.ReadingFromSource

    Dim burningThread As New Thread(AddressOf StartBurnThread)
    burningThread.Start(Me.imageFileName)
  End Sub

  Protected Overrides Sub SetBurnProcessStartedText()
    lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Disc copy started."))
    lblStatus.Text = String.Format(RM.GetString("Status: Extracting from drive '{0}'..."), cbSourceDevice.SelectedDeviceLetter)
  End Sub

  Protected Overrides Sub HandleBurnProcessStarted()
    MyBase.HandleBurnProcessStarted()

    If Animation.FirstFrame <> 105 Then '+1
      Animation.FirstFrame = 105 '+1
      Animation.LastFrame = 123 '+1
      Animation.Delay = 100
      Animation.Pause = 150
      Animation.StartAnimation()
    End If
  End Sub

  Private Sub StartBurnThread(ByVal targetPath As Object)
    Try
      HandleBurnProcessStarted()
      Dim grabInfo As Dictionary(Of Int32, CDBurnerXP.Burning.Grabber.GrabIsoInfo) = Nothing

      If Me.burnAfterGrabbing Then
        grabInfo = If(Me.cachedGrabInfo Is Nothing, grabber.GrabDiscAsMultipleTracks(CStr(targetPath)), Me.cachedGrabInfo)
        Me.cachedGrabInfo = grabInfo
      ElseIf Me.targetImageType = 0 Then
        grabber.GrabDisc(CStr(targetPath))
      Else
        grabber.GrabTrackAsData(1, CStr(targetPath))
      End If

      If Me.burnAfterGrabbing AndAlso Not m_BurningProcessCancelled Then
        ' Set ImageBurner data
        m_Status = Status.Burning
        Dim ok As Boolean = CBool(Me.Invoke(New PrepareBurnDelegate(AddressOf PrepareBurn), grabInfo))
        If (ok) Then
          imageBurner.Burn(grabInfo)
        End If
      End If

    Catch ex As COMException
      Me.Invoke(New ParameterizedThreadStart(AddressOf HandleError), ex)
    Finally
      Me.BeginInvoke(New MethodInvoker(AddressOf StopWrite))
    End Try
  End Sub

  Private Function PrepareBurn(ByVal grabDiscInfo As Dictionary(Of Int32, CDBurnerXP.Burning.Grabber.GrabIsoInfo)) As Boolean
    tmrElapsed.Stop()

    ' After ripping a disc, we obviously need to eject before proceeding if source == target
    If cbSourceDevice.SelectedDeviceLetter = cbTargetDevice.DeviceSelect.SelectedDeviceLetter Then
      cbTargetDevice.DeviceSelect.SelectedStarBurnDevice.Eject()
    End If

    ' Calculate the size of all tracks
    Dim sizeInBytes As Long = 0
    For Each track As KeyValuePair(Of Int32, CDBurnerXP.Burning.Grabber.GrabIsoInfo) In grabDiscInfo
      sizeInBytes += FormatFileSize.BlocksToDataBytes(track.Value.TrackSizeLB)
    Next

    If Not CheckDisc.Show(Me, cbTargetDevice.DeviceSelect.SelectedStarBurnDevice, CheckDisc.DiscFeature.Empty, CheckDisc.DiscType.Any, sizeInBytes, False, False) Then
      m_BurningProcessCancelled = True
      Return False
    End If

    lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Started burning the extracted image to disc."))

    imageBurner.Drive = cbTargetDevice.DeviceSelect.SelectedStarBurnDevice
    imageBurner.NextSessionAllowed = Not chkFinalize.Checked
    imageBurner.ImageFileName = Me.imageFileName
    imageBurner.WriteSpeed = cbTargetDevice.WriteSpeedLongValue

    MediaNotifier.Enabled = False
    tmrElapsed.Start()
    Return True
  End Function

  Private Sub StopWrite()
    tmrElapsed.Enabled = False
    prgProgress.ShowMarquee = False
    lblStatus.Text = RM.GetString("Status: Idle")
    Animation.StopAnimation()

    If Not m_BurningProcessCancelled AndAlso grabber.LastErrorCode = STARBURN_RESULT_CODES.STARBURN_OPERATION_SUCCEEDED AndAlso imageBurner.LastErrorCode = STARBURN_RESULT_CODES.STARBURN_OPERATION_SUCCEEDED Then
      If cbHardDisc.Checked Then
        lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Successfully extracted ISO image."))
        Dim deltaTime As TimeSpan = Utility.TimeSpanSeconds(m_ElapsedTime - m_StartTime)
        MessageBox.Show(Me, String.Format(RM.GetString("Extracting ISO image from '{0}' to '{1}' successfully completed in {2}(hh:mm:ss)"), cbSourceDevice.SelectedStarBurnDevice.DriveInfo.Letter, Path.GetFileName(txtImagePath.Text), deltaTime.ToString()), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1)
      Else
        Dim furtherCopies As Boolean = FurtherCopiesRequired()

        If furtherCopies OrElse (chkEject.Checked AndAlso Not Me.imageBurner.Drive Is Nothing) Then
          TryEject(Me.imageBurner.Drive)
        End If

        lvwStatus.AddItem(LogListView.LogType.Information, RM.GetString("Copying disc was completed successfully."))
        If furtherCopies Then
          Me.BeginBurnNextDisc()
          Exit Sub
        Else
          Me.cachedGrabInfo = Nothing
          Dim deltaTime As TimeSpan = Utility.TimeSpanSeconds(m_ElapsedTime - m_StartTime)
          Dim msgboxstr As String = String.Format(RM.GetString("Copying disc was completed successfully in {0}(hh:mm:ss)."), deltaTime.ToString())
          MessageBox.Show(Me, msgboxstr, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information)
        End If
      End If
    ElseIf m_BurningProcessCancelled Then
      If m_Status = Status.Burning Then
        lvwStatus.AddItem(LogListView.LogType.Warning, RM.GetString("Cancelled burning image to disc."))
      Else
        lvwStatus.AddItem(LogListView.LogType.Warning, RM.GetString("ISO extraction was cancelled."))
      End If
    End If

#If Not Debug Then
    If Not cbHardDisc.Checked then
      PathEx.TryDeleteDirectory(me.imageFileName)
    End If
#End If

    m_Status = Status.Idle
    prgProgress.Value = 0
    Me.Text = RM.GetString("Copy Disc.TitleBar")
    ResetUiAfterWork()
  End Sub

  Private Sub cboSourceDevice_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cbSourceDevice.SelectedIndexChanged
    cmdStartBurn.Enabled = Not cbSourceDevice.SelectedStarBurnDevice Is Nothing
    If Not cbSourceDevice.SelectedStarBurnDevice Is Nothing Then
      lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Source device changed to: {0}"), cbSourceDevice.SelectedItem.ToString()))
    End If
  End Sub

  Private Sub OnTargetChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cbTargetDevice.SelectedIndexChanged
    If cbDevice.Checked Then
      If Not cbTargetDevice.DeviceSelect.SelectedDevice Is Nothing Then
        lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Target device changed to: {0}"), cbTargetDevice.DeviceSelect.SelectedItem.ToString()))
      End If
    Else
      lvwStatus.AddItem(LogListView.LogType.Information, String.Format(RM.GetString("Target device changed to: {0}"), RM.GetString("Hard Disc")))
    End If
  End Sub

  Protected Overrides Sub OnClosing(ByVal e As System.ComponentModel.CancelEventArgs)
    MyBase.OnClosing(e)

    If m_Status = Status.ReadingFromSource Then
      Me.grabber.Cancel()
      m_BurningProcessCancelled = True
      lvwStatus.AddItem(LogListView.LogType.Warning, RM.GetString("Cancelled extracting image."))
      e.Cancel = True
    ElseIf m_Status = Status.Burning Then
      m_BurningProcessCancelled = True
      lvwStatus.AddItem(LogListView.LogType.Warning, RM.GetString("Burning was cancelled by the user, aborting please wait"))
      imageBurner.Cancel()
      e.Cancel = True
    End If

    m_AutoImportEnabled = True
  End Sub

  Private Sub cmdSelectPath_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSelectPath.Click
    With ISOPathDialog
      .AddExtension = True
      If cboImageType.SelectedIndex = 0 Then
        .DefaultExt = ".mds"
        .Filter = RM.GetString("Media Descriptor File") & " (*.mds)|*.mds|" & RM.GetString("All files") & " (*.*)|*.*"
      Else
        .DefaultExt = ".iso"
        .Filter = RM.GetString("ISO File") & " (*.iso)|*.iso|" & RM.GetString("All files") & " (*.*)|*.*"
      End If

      .InitialDirectory = ApplicationSettings.DataCopyImagePath
      If .ShowDialog(Me) = DialogResult.Cancel Then Exit Sub
      txtImagePath.Text = .FileName
      ApplicationSettings.DataCopyImagePath = Path.GetDirectoryName(.FileName)
    End With
  End Sub

  Protected Overrides Sub OnCancelClick(ByVal sender As Object, ByVal e As System.EventArgs)
    MyBase.OnCancelClick(sender, e)

    Me.Close()
  End Sub

  Protected Overrides Sub OnClosed(ByVal e As System.EventArgs)
    MyBase.OnClosed(e)
    RemoveHandler grabber.OnProgressExtended, AddressOf OnGrabberProgress
    RemoveHandler imageBurner.OnProgress, AddressOf OnImageBurnerProgress

    ApplicationSettings.CopyDisableHardwareEC = chkDisableHardwareEC.Checked
    ApplicationSettings.CopyEjectDisc = chkEject.Checked
    ApplicationSettings.CopyFinaliseDisc = chkFinalize.Checked
    ApplicationSettings.CopyReadRetries = Convert.ToInt32(numReadRetries.Value)
    ApplicationSettings.CopyIgnoreUnreadableData = chkIgnoreUnreadable.Checked
    ApplicationSettings.CopyImageType = cboImageType.SelectedIndex
    ApplicationSettings.CopySourceDevice = cbSourceDevice.SelectedDeviceLetter

    Dispose(True)
  End Sub

  Private Sub cbDevice_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbDevice.CheckedChanged
    ActivateControls(True)
    OnTargetChanged(sender, e)
  End Sub

  Private Sub cbHardDisc_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbHardDisc.CheckedChanged
    ActivateControls(True)
    OnTargetChanged(sender, e)
  End Sub

  Private Sub cboImageType_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboImageType.SelectedIndexChanged
    If txtImagePath.Text.EndsWith(".iso") OrElse txtImagePath.Text.EndsWith(".mds") Then
      If cboImageType.SelectedIndex = 0 Then
        txtImagePath.Text = txtImagePath.Text.Remove(txtImagePath.Text.Length - 3, 3) & "mds"
      Else
        txtImagePath.Text = txtImagePath.Text.Remove(txtImagePath.Text.Length - 3, 3) & "iso"
      End If
    End If
  End Sub
End Class
