'*************************************************************************
' Klass: frmCDDVDImportSession
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-24 20:01
' Kommentarer: 
'*************************************************************************

Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports CDBurnerXP.Burning
Imports CDBurnerXP.Controls

Public Class frmCDDVDImportSession

  Private oDevice As NMSDVDXLib.DeviceX
  Private m_LayoutManager As FileLayoutManager
  Private m_lUsedSpace As Long = 0
  Private m_lFreeSpace As Long = 0
  Private m_lTotalSpace As Long = 0
  Private m_iNumberOfSessions As Integer = 0
  Private m_sFileSystem As String
  Private m_SessionImported As Boolean = False
  Private intSelectedSession As Short = 0

  <CLSCompliant(False)> Public Sub New(ByVal layoutManager As FileLayoutManager, ByRef oDeviceX As NMSDVDXLib.DeviceX)
    MyBase.New()

    'This call is required by the Windows Form Designer.
    InitializeComponent()
    AcceptButton = cmdImportSession
    CancelButton = cmdCancel

    'Add any initialization after the InitializeComponent() call
    oDevice = oDeviceX
    m_LayoutManager = layoutManager
  End Sub

  Public ReadOnly Property IsImported() As Boolean
    Get
      Return m_SessionImported
    End Get
  End Property

  Private Sub frmCDDVDSession_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
    If oDevice Is Nothing Then
      Me.Close()
      MessageBox.Show(RM.GetString("The drive(s) could not be initialized..."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      Exit Sub
    End If

    'Language Set
    Me.Text = RM.GetString("Import Session")
    lblDiscStatus.Text = RM.GetString("Disc Status:")
    lblCDDiscType.Text = RM.GetString("CD Disc Type:")
    lblMediaType.Text = RM.GetString("Media Type:")
    lblSessions.Text = RM.GetString("Number of Session:")
    lblLastSessionStatus.Text = RM.GetString("Last Session Status:")
    lblIsOpen.Text = RM.GetString("Open:")
    lblSelect.Text = RM.GetString("Selact a track or session to continue with multisession writing")
    cmdImportSession.Text = RM.GetString("Import Session")
    cmdCancel.Text = RM.GetString("Cancel")
    sepSessionView.Text = RM.GetString("Disc content")

    PopulateDiscInfo()
  End Sub

  Protected Overrides Sub OnMediaChanged(ByVal VolumeLabel As String)
    PopulateDiscInfo()
  End Sub

  Public Function ImportSession() As Boolean
    Me.Cursor = Cursors.WaitCursor
    'Refresh disc info
    oDevice.Refresh(False)

    cmdImportSession.Enabled = False
    cmdCancel.Enabled = False

    'Check for media loaded
    If oDevice.GetDiscInfo.GetMediaType() = NMSDVDXLib.eMediaTypeX.mtNOT_LOADED Then
      MessageBox.Show(Me, RM.GetString("Please insert a disc before continuing!"), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      Me.Cursor = Cursors.Default
      cmdCancel.Enabled = True
      Return False
    End If

    If m_LayoutManager.ImportFromDisc(oDevice, intSelectedSession) = False Then
      Dim strImportError As String = RM.GetString(BurningEngine.GetImportErrorMsg(m_LayoutManager.DataImage.GetImportError))
      MessageBox.Show(Me, RM.GetString("Session could not be imported.") & " " & strImportError, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      m_LayoutManager.Clear()
      m_SessionImported = False
      Me.Cursor = Cursors.Default
      cmdCancel.Enabled = True
      Return False
    Else
      m_SessionImported = True
      Me.Cursor = Cursors.Default
      cmdCancel.Enabled = True
      Return True
    End If
  End Function

  Private Sub PopulateDiscInfo()
    Dim objDiscInfo As NMSDVDXLib.DiscInfoX

    'Clear the nodes
    tvwSessions.Nodes.Clear()

    'Refresh w/ Logical space being calculated as well
    'Logical space is different than physical space with
    'some formats as DVD+RW
    oDevice.Refresh(True)

    'Get disc info from device
    objDiscInfo = oDevice.GetDiscInfo()

    'Get the Disc Status
    Select Case objDiscInfo.GetDiscStatus()
      Case NMSDVDXLib.eDiscStatusX.dsEmpty
        lblValueDiscStatus.Text = RM.GetString("Empty Disc")
      Case NMSDVDXLib.eDiscStatusX.dsIncomplete
        lblValueDiscStatus.Text = RM.GetString("Incomplete Disc")
      Case NMSDVDXLib.eDiscStatusX.dsComplete
        lblValueDiscStatus.Text = RM.GetString("Completed Disc")
      Case NMSDVDXLib.eDiscStatusX.dsUnknown 'Fall through
      Case Else
        lblValueDiscStatus.Text = RM.GetString("Unknown Status")
    End Select

    'Get the CD Disc Type (CD only)
    Select Case objDiscInfo.GetCDDiscType()
      Case NMSDVDXLib.eCDDiscTypeX.dtCddaCDRom
        lblValueCDDiscType.Text = RM.GetString("CDROM / CDDA Format")
      Case NMSDVDXLib.eCDDiscTypeX.dtCDI
        lblValueCDDiscType.Text = RM.GetString("CDI Format")
      Case NMSDVDXLib.eCDDiscTypeX.dtCDRomXA
        lblValueCDDiscType.Text = RM.GetString("CDROM XA Format")
      Case NMSDVDXLib.eCDDiscTypeX.dtUnknown 'Fall through
        lblValueCDDiscType.Text = RM.GetString("Unknown")
      Case Else
        lblValueCDDiscType.Text = RM.GetString("N/A")
    End Select

    'Get the Media Type
    lblValueMediaType.Text = CDBurnerXP.Burning.BurningEngine.GetDiscDescription(objDiscInfo.GetMediaType())

    'Get the Last session's Status
    Select Case objDiscInfo.GetLastSessionStatus()
      Case NMSDVDXLib.eSessionStatusX.ssEmpty
        lblValueLastSessionStatus.Text = RM.GetString("Empty Session")
      Case NMSDVDXLib.eSessionStatusX.ssIncomplete
        lblValueLastSessionStatus.Text = RM.GetString("Incomplete Session")
      Case NMSDVDXLib.eSessionStatusX.ssReserved
        lblValueLastSessionStatus.Text = RM.GetString("Reserved Session")
      Case NMSDVDXLib.eSessionStatusX.ssComplete
        lblValueLastSessionStatus.Text = RM.GetString("Complete Session")
      Case Else
        lblValueLastSessionStatus.Text = RM.GetString("Status Unknown")
    End Select

    'Get the session count
    lblValueSessions.Text = objDiscInfo.GetSessionCount().ToString()

    'Is Open - Can Append (OPEN)
    If objDiscInfo.IsOpen() = True Then
      lblValueIsOpen.Text = RM.GetString("Yes")
    Else
      lblValueIsOpen.Text = RM.GetString("No")
    End If

    PopulateTrackInfo()
  End Sub

  Private Sub PopulateTrackInfo()
    Dim oDiscInfo As NMSDVDXLib.DiscInfoX
    Dim oTrackInfo As NMSDVDXLib.TrackInfoX
    Dim oToc As NMSDVDXLib.TocX
    Dim oSessionInfo As NMSDVDXLib.SessionInfoX
    Dim intSession As Short
    Dim intCurrSession As Short
    Dim nodDisc As TreeNode = Nothing
    Dim dblSpaceBytes As Double
    Dim sessionnode As TreeNode = Nothing
    Dim intTrack As Short
    Dim strTemp As String
    Dim lVolTypes As Integer
    Dim numTracks As Short

    oTrackInfo = New NMSDVDXLib.TrackInfoX
    oToc = New NMSDVDXLib.TocX
    oSessionInfo = New NMSDVDXLib.SessionInfoX


    intSession = 0
    intCurrSession = 1
    If oDevice.GetDiscInfo.GetMediaType() = NMSDVDXLib.eMediaTypeX.mtNOT_LOADED Then
      nodDisc = tvwSessions.Nodes.Add(RM.GetString("There is no disc in the drive, please insert a writable disc"))
      nodDisc.ImageIndex = 4
      nodDisc.SelectedImageIndex = 4
      nodDisc.Tag = -1
      cmdImportSession.Enabled = False
      cmdImportSession.Text = RM.GetString("Import Session")
    Else
      'We will display all tracks - by session
      '...and tracks within those sesions
      'Get disc info..already refreshed from above
      oDiscInfo = oDevice.GetDiscInfo()

      'Insert root of our tree
      If oDiscInfo.IsOpen Then
        nodDisc = tvwSessions.Nodes.Add("DISC (" & lblValueMediaType.Text & ")")
        nodDisc.ImageIndex = 0
        nodDisc.SelectedImageIndex = 0
        nodDisc.Tag = 0
      Else
        nodDisc = tvwSessions.Nodes.Add(RM.GetString("This disc is not writable, please try another disc"))
        nodDisc.ImageIndex = 3
        nodDisc.SelectedImageIndex = 3
        nodDisc.Tag = -1
        cmdImportSession.Enabled = False
        cmdImportSession.Text = RM.GetString("Import Session")
        Exit Sub
      End If

      numTracks = BurningEngine.GetLastTrackNumber(oDiscInfo)

      'Sessions and Tracks are 1 based index(not zero)
      'Spin through the tracks and sessions
      For intTrack = 1 To numTracks
        Dim tracknode As New TreeNode
        'Try to read this Session's TOC
        If oDevice.ReadTrackInfo(oTrackInfo, intTrack) = False Then
          Exit For 'No track data
        End If

        'Get the session of the track
        intCurrSession = oTrackInfo.GetSessionNumber()

        'Add session to the tree?
        If intSession <> intCurrSession Then

          intSession = intCurrSession
          sessionnode = New TreeNode
          dblSpaceBytes = Math.Ceiling((CDbl(oTrackInfo.GetSizeBlocks) * 2048) / (1024 * 1024))
          'Format data..this is for display only
          strTemp = RM.GetString("Session") & String.Format(" {0:0#} ({1:### ###} MB)", intSession, dblSpaceBytes)

          'Add to tree
          sessionnode = nodDisc.Nodes.Add(strTemp)
          sessionnode.ImageIndex = 1
          sessionnode.SelectedImageIndex = 1
          sessionnode.Tag = intSession
        End If

        'Get some additional info
        strTemp = RM.GetString("Track") & String.Format(" {0:0#}", intTrack)
        strTemp = strTemp & " - LBA:" & String.Format("{0:0#######}", oTrackInfo.GetStartLBA())
        strTemp = strTemp & " " & RM.GetString("Size:") & String.Format("{0:0#######}", oTrackInfo.GetSizeBlocks())

        'Audio or Data?
        If oTrackInfo.IsAudio() = True Then
          strTemp = strTemp & "(" & RM.GetString("Audio") & ")"
        Else
          strTemp = strTemp & "(" & RM.GetString("Data") & ")"

          'We will also get all the volumes for this track
          'Get the detected volume types for this track
          lVolTypes = oDevice.GetLogicalVolumeTypes(intTrack)

          'Check returned mask for detected Volume types
          If (lVolTypes And NMSDVDXLib.eVolumeTypeX.vtISO9660) > 0 Then strTemp = strTemp & " ISO9660 "
          If (lVolTypes And NMSDVDXLib.eVolumeTypeX.vtJoliet) > 0 Then strTemp = strTemp & " JOLIET "
          If (lVolTypes And NMSDVDXLib.eVolumeTypeX.vtUDF) > 0 Then strTemp = strTemp & " UDF "
        End If

        'Is Blank(empty) Track?
        If oTrackInfo.IsBlank() = True Then
          strTemp = strTemp & " " & RM.GetString("Empty") & " "
        End If

        'Add info to the tree
        tracknode = sessionnode.Nodes.Add(strTemp)
        tracknode.ImageIndex = 2
        tracknode.SelectedImageIndex = 2
        tracknode.Tag = intSession
      Next

      'Expand the tree
      Call tvwSessions.ExpandAll()
    End If
  End Sub

  Private Sub cmdCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCancel.Click
    Me.Close()
  End Sub

  Private Sub cmdImportSession_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdImportSession.Click
    ImportSession()
    Me.Close()
  End Sub

  Private Sub tvwSessions_AfterSelect(ByVal sender As System.Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles tvwSessions.AfterSelect
    intSelectedSession = Convert.ToInt16(tvwSessions.SelectedNode.Tag)
    If intSelectedSession > 0 Then
      cmdImportSession.Text = String.Format(RM.GetString("Import Session {0:0#}"), intSelectedSession)
      cmdImportSession.Enabled = True
    Else
      cmdImportSession.Text = RM.GetString("Import Session")
      cmdImportSession.Enabled = False
    End If
  End Sub

  Private Sub tvwSessions_DoubleClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tvwSessions.DoubleClick
    cmdImportSession.PerformClick()
  End Sub
End Class
