'*************************************************************************
' Klass: frmLanguageSelect
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-26 08:29
' Kommentarer: 
'*************************************************************************

Imports CDBurnerXP.Forms
Imports System.Globalization
Imports System.IO
Imports CDBurnerXP

Public Class frmLanguageSelect

  Private m_Language As String = ""
  Private m_AvailableLanguages As List(Of CultureInfo)

  Public ReadOnly Property Language() As String
    Get
      Return m_Language
    End Get
  End Property

  Public ReadOnly Property AvailableLanguages() As CultureInfo()
    Get
      If m_AvailableLanguages Is Nothing Then
        m_AvailableLanguages = New List(Of CultureInfo)

        Try
          Dim ks As New DirectoryInfo(Application.StartupPath)
          For Each Di As DirectoryInfo In ks.GetDirectories
            For Each strLang As CultureInfo In System.Globalization.CultureInfo.GetCultures(CultureTypes.FrameworkCultures)
              If Di.Name = strLang.Name Then
                m_AvailableLanguages.Add(strLang)
              End If
            Next
          Next
        Catch ex As IOException
          ' System.IO.IOException: There are no more files.
          ' Should not occur, but some cases were reported. 
          ' Nicely fall back to the default language.
        Catch ex As UnauthorizedAccessException
          ' Should not occur, but some cases were reported. 
          ' Nicely fall back to the default language.
        End Try
      End If

      Return m_AvailableLanguages.ToArray()
    End Get
  End Property

  Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
    MyBase.OnLoad(e)

    For Each strLang As CultureInfo In AvailableLanguages
      cboLanguage.Items.Add(New TaggedComboBoxItem(strLang.NativeName, strLang.Name))
    Next

    cboLanguage.Items.Add(New TaggedComboBoxItem("Default (English)", "en-US"))
    cboLanguage.SelectedIndex = cboLanguage.Items.Count - 1
    ' Preselect current language, if possible
    Dim curLang As String = CultureInfo.CurrentCulture.NativeName
    For i As Integer = 0 To cboLanguage.Items.Count - 1 Step 1
      If cboLanguage.Items(i).ToString() = curLang Then
        cboLanguage.SelectedIndex = i
      End If
    Next
  End Sub

  Private Sub cboLanguage_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboLanguage.SelectedIndexChanged
    m_Language = CStr(DirectCast(cboLanguage.Items.Item(cboLanguage.SelectedIndex), TaggedComboBoxItem).Tag)
  End Sub

  Private Sub cmdDone_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdDone.Click
    Me.Close()
  End Sub

End Class
