'*************************************************************************
' Klass: frmOptions
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-26 08:31
' Kommentarer: 
'*************************************************************************

Imports System.IO
Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports CDBurnerXP.IO
Imports CDBurnerXP.Burning
Imports CDBurnerXP.Forms
Imports CDBurnerXP
Imports CDBurnerXP.Net
Imports Microsoft.Win32

Public Class frmOptions

  Private messageBoxes As String() = New String() {"AudioTracksSorted", "AudioCancelled", "AudioCompleted", "SaveAudioCompilation", "ClearAudio", "BeforeErasing", "AfterErase", "DataCompleted", "DataCancelled", "DataCompletedVerifyCancelled", "DataCompletedVerifyOK", "AddISO", "AddPlaylist", "SaveDataCompilation", "ClearData", "IsoCompleted", "IsoCancelled"}
  Private defaultLangItem As New TaggedComboBoxItem("English (United States)", "en-US")

  Private Sub frmOptions_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
    AcceptButton = cmdOK
    CancelButton = cmdCancel

    'Language
    Me.Text = RM.GetString("Options")
    tabGeneral.Text = RM.GetString("General Settings")
    tabData.Text = RM.GetString("Data Options")
    tabAudio.Text = RM.GetString("Audio Options")
    sepLanguage.Text = RM.GetString("Language:")
    lblLanguageSelect.Text = RM.GetString("Select Language:")
    lblNote.Text = RM.GetString("Note: CDBurnerXP need to be restarted to fully apply this setting.")
    chkCustomTempFolder.Text = RM.GetString("Custom folder for &temporary data:")
    sepMiscSettings.Text = RM.GetString("Misc. Settings:")
    sepMiscSettingsData.Text = RM.GetString("Misc. Settings:")
    cmdOK.Text = RM.GetString("OK")
    cmdApply.Text = RM.GetString("Apply")
    cmdCancel.Text = RM.GetString("Cancel")
    cmdDefault.Text = RM.GetString("Default Settings")
    sepDiscLabelSettings.Text = RM.GetString("Disc Label Settings:")
    lblDefaultDiscLabel.Text = RM.GetString("Default disc label:")
    lblDate.Text = RM.GetString("Variables:")
    lblExample.Text = RM.GetString("Example:")
    lblInfo.Text = RM.GetString("Remember that labels will be truncated to 16 characters.")
    chkExitToNotify.Text = RM.GetString("Exit to notify panel")
    chkAskForDataType.Text = RM.GetString("Ask me how to burn a data disc")
    chkUpdateAutomatically.Text = RM.GetString("&After opening a compilation, update it automatically")
    chkStartLastUsed.Text = RM.GetString("Remember last used project type")
    lblReplayGain.Text = RM.GetString("If Replay Gain information is attached to your audio files, CDBurnerXP can apply the volume change automatically before burning.")
    cboNoReplayGain.Text = RM.GetString("Do &not apply Replay Gain")
    cboTrackGain.Text = RM.GetString("Use &Track Gain, or Album Gain if not available")
    cboAlbumGain.Text = RM.GetString("Use &Album Gain, or Track Gain if not available")
    chkEnableLogging.Text = RM.GetString("Enable logging (only for diagnostic purposes)")
    chkCacheAutomatically.Text = RM.GetString("Automatically cache files from removable media")
    chkUseSingleUnit.Text = RM.GetString("&Only use a single unit for file sizes")
    chkAutoUpdate.Text = RM.GetString("A&utomatically download and install updates")
    lblMessageBoxes.Text = RM.GetString("Show &message boxes for the following events:")
    chkAutoCDText.Text = RM.GetString("&Automatically generate CD-Text information from tags (ID3, APE)")
    chkIgnoreFileRenames.Text = RM.GetString("&Do not show file rename requirements caused by file system limitations")
    chkCacheNetworkAutomatically.Text = RM.GetString("Automatically cache files from &network locations")

    ' Fill message boxes
    For Each name As String In Me.messageBoxes
      Dim index As Integer = chklMessageBoxes.Items.Add(New TaggedComboBoxItem(RM.GetString("MsgBox." & name), name))
      chklMessageBoxes.SetItemChecked(index, Conversion.ToBoolean(Settings.GetValue("MessageBoxes", name, True)))
    Next

    LoadSettings()
  End Sub


  Private Sub ApplySettings()
    'General options
    If Not cboLanguage.SelectedItem Is Nothing Then
      ' Error report case
      ApplicationSettings.AppLanguage = CStr(DirectCast(cboLanguage.SelectedItem, TaggedComboBoxItem).Tag)
    End If
    ApplicationSettings.CustomTempFolder = txtTempPath.Text
    ApplicationSettings.UseCustomTempFolder = chkCustomTempFolder.Checked
    ApplicationSettings.ExitToNotify = chkExitToNotify.Checked
    ApplicationSettings.LoggingEnabled = chkEnableLogging.Checked
    ApplicationSettings.DataCacheFilesAutomatically = chkCacheAutomatically.Checked
    ApplicationSettings.DataCacheNetworkFilesAutomatically = chkCacheNetworkAutomatically.Checked
    ApplicationSettings.DataUpdateAutomatically = chkUpdateAutomatically.Checked
    ApplicationSettings.AutoUpdate = chkAutoUpdate.Checked
    ApplicationSettings.AudioAutoCDText = chkAutoCDText.Checked
    ApplicationSettings.IgnoreFileRenameRequirements = chkIgnoreFileRenames.Checked

    ApplicationSettings.DataAskForSettings = chkAskForDataType.Checked

    ApplicationSettings.StartLastUsedProjectType = chkStartLastUsed.Checked
    ApplicationSettings.OnlyUseSingleUnit = chkUseSingleUnit.Checked
    FormatFileSize.UseSingleUnit = ApplicationSettings.OnlyUseSingleUnit

    'Audio options
    If Directory.Exists(PathEx.GetTempRipPath()) Then
      Try
        For Each fil As String In Directory.GetFiles(PathEx.GetTempRipPath())
          File.Delete(fil)
        Next
      Catch ex As Exception
        ' do not care, as always
      End Try
    End If

    ' Replay Gain
    If cboNoReplayGain.Checked Then ApplicationSettings.AudioReplayGain = CByte(CDBurnerXP.IO.ReplayGainOption.None)
    If cboTrackGain.Checked Then ApplicationSettings.AudioReplayGain = CByte(CDBurnerXP.IO.ReplayGainOption.Track)
    If cboAlbumGain.Checked Then ApplicationSettings.AudioReplayGain = CByte(CDBurnerXP.IO.ReplayGainOption.Album)

    'Data options
    ApplicationSettings.DataLabel = txtDefaultDiscLabel.Text

    ApplicationSettings.LoggingEnabled = chkEnableLogging.Checked

    ' Fill message boxes
    For i As Integer = 0 To chklMessageBoxes.Items.Count - 1
      Dim item As TaggedComboBoxItem = CType(chklMessageBoxes.Items(i), TaggedComboBoxItem)
      Settings.SetValue("MessageBoxes", CStr(item.Tag), chklMessageBoxes.GetItemChecked(i))
      Settings.SetValue("MessageBoxes", "Result" + CStr(item.Tag), DialogResult.Yes)
    Next

    Try
      ' Try to save settings. Fails if no write permission in user folder. Rare, but shit happens.
      ApplicationSettings.Save()
    Catch ex As Exception
      MessageBox.Show(RM.GetString("Application settings can not be saved:") & " " & ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
    End Try
  End Sub

  Public Sub LoadSettings()
    'General options
    Dim ks As New DirectoryInfo(Application.StartupPath)
    Dim cultures As CultureInfo() = System.Globalization.CultureInfo.GetCultures(System.Globalization.CultureTypes.FrameworkCultures)

    For Each Di As DirectoryInfo In ks.GetDirectories
      For Each strLang As System.Globalization.CultureInfo In cultures
        If Di.Name = strLang.Name Then
          cboLanguage.Items.Add(New TaggedComboBoxItem(strLang.NativeName, strLang.Name))
        End If
      Next
    Next
    cboLanguage.Items.Add(defaultLangItem)
    Dim index As Integer = 0
    Dim strLang2 As String = ApplicationSettings.AppLanguage
    For Each itm As TaggedComboBoxItem In cboLanguage.Items
      If CStr(itm.Tag) = strLang2 Then
        Exit For
      End If
      index += 1
    Next
    If index > cboLanguage.Items.Count - 1 Then
      cboLanguage.SelectedIndex = 0
    Else
      cboLanguage.SelectedIndex = index
    End If

    txtTempPath.Text = ApplicationSettings.CustomTempFolder
    chkCustomTempFolder.Checked = ApplicationSettings.UseCustomTempFolder

    chkExitToNotify.Checked = ApplicationSettings.ExitToNotify
    chkEnableLogging.Checked = ApplicationSettings.LoggingEnabled
    chkAskForDataType.Checked = ApplicationSettings.DataAskForSettings
    chkStartLastUsed.Checked = ApplicationSettings.StartLastUsedProjectType
    chkUseSingleUnit.Checked = ApplicationSettings.OnlyUseSingleUnit
    chkCacheAutomatically.Checked = ApplicationSettings.DataCacheFilesAutomatically
    chkCacheNetworkAutomatically.Checked = ApplicationSettings.DataCacheNetworkFilesAutomatically
    chkUpdateAutomatically.Checked = ApplicationSettings.DataUpdateAutomatically
    chkAutoCDText.Checked = ApplicationSettings.AudioAutoCDText
    chkIgnoreFileRenames.Checked = ApplicationSettings.IgnoreFileRenameRequirements

    If Not AutoUpdate.IsOnlineUpdateAllowed Then
      chkAutoUpdate.Enabled = False
      chkAutoUpdate.Checked = False
    Else
      chkAutoUpdate.Checked = ApplicationSettings.AutoUpdate
    End If

    ' Replay gain
    cboTrackGain.Checked = (ApplicationSettings.AudioReplayGain = CDBurnerXP.IO.ReplayGainOption.Track)
    cboAlbumGain.Checked = (ApplicationSettings.AudioReplayGain = CDBurnerXP.IO.ReplayGainOption.Album)
    cboNoReplayGain.Checked = (ApplicationSettings.AudioReplayGain = CDBurnerXP.IO.ReplayGainOption.None)

    'Data Options
    txtDefaultDiscLabel.Text = ApplicationSettings.DataLabel
  End Sub

  Private Sub cmdApply_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdApply.Click
    ApplySettings()
  End Sub

  Private Sub cmdOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdOK.Click
    ApplySettings()
    Me.Close()
  End Sub

  Private Sub cmdDefault_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdDefault.Click
    If MessageBox.Show(RM.GetString("Are you sure that you want to reset the settings for CDBurnerXP?"), Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes Then
      'Tab general options
      cboLanguage.SelectedItem = defaultLangItem

      txtTempPath.Text = String.Empty

      chkExitToNotify.Checked = False
      chkEnableLogging.Checked = False
      chkAskForDataType.Checked = True
      chkStartLastUsed.Checked = False
      cboNoReplayGain.Checked = True
      chkUseSingleUnit.Checked = False
      chkCacheAutomatically.Checked = True
      chkUpdateAutomatically.Checked = False
      chkCustomTempFolder.Checked = False
      chkAutoCDText.Checked = True
      chkIgnoreFileRenames.Checked = False
      chkCacheNetworkAutomatically.Checked = False

      If AutoUpdate.IsOnlineUpdateAllowed Then
        chkAutoUpdate.Checked = True
      End If

      'Tab data options
      txtDefaultDiscLabel.Text = "Disc"

      ' Fill message boxes
      For i As Integer = 0 To chklMessageBoxes.Items.Count - 1
        chklMessageBoxes.SetItemChecked(i, True)
      Next

    End If
  End Sub

  'Data Options Section
  Private Sub txtDefaultDiscLabel_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtDefaultDiscLabel.TextChanged
    Dim truncText As String = DataCompilation.ReplaceDiscLabelVariables(txtDefaultDiscLabel.Text)
    If truncText.Length > 16 Then
      truncText = truncText.Remove(16, truncText.Length - 16)
    End If
    lblExample.Text = RM.GetString("Example:") & " " & truncText
  End Sub


  Private Sub cboLanguage_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cboLanguage.SelectedIndexChanged
    g_sLanguage = DirectCast(cboLanguage.SelectedItem, TaggedComboBoxItem).Tag
  End Sub


  Private Sub chkCustomTempFolder_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkCustomTempFolder.CheckedChanged
    txtTempPath.Enabled = chkCustomTempFolder.Checked
    cmdTempPath.Enabled = chkCustomTempFolder.Checked
  End Sub

  Private Sub cmdTempPath_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdTempPath.Click
    With dlgFolder
      .Description = RM.GetString("Select folder...")
      .SelectedPath = txtTempPath.Text
      If dlgFolder.ShowDialog(Me) = DialogResult.OK Then
        txtTempPath.Text = .SelectedPath
      End If
    End With
  End Sub
End Class
