﻿using System;
using System.Collections.Generic;
using System.Text;
using CDBurnerXP.Forms;
using CDBurnerXP.IO;

namespace CDBurnerXP.Burning
{
    /// <summary>
    /// Provides information about a specific disc type.
    /// </summary>
    public class DiscType
    {
        /// <summary>
        ///  http://www.osta.org/technology/cdqa5.htm
        ///  Measured as audio speed (2,352 Bytes/Block)
        /// </summary>
        public const double CDBaseSpeedInKBs = 176.5;

        public const int DVDBaseSpeedInKBs = 1385;

        public const int BDBaseSpeedInKBs = 4500;

        #region Predefined disc types

        public static DiscType DVDDL
        {
            get
            {
                return new DiscType(String.Format(BaseForm.RM.GetString("{0} Dual Layer ({1})"), "DVD DL", FormatFileSize.FormatDecimal(8500000000)), 8543666176);
            }
        }

        public static DiscType DVD
        {
            get
            {
                // DVD-R/DVD+R has a minor size difference.
                return new DiscType(String.Format(BaseForm.RM.GetString("{0} Single Layer ({1})"), "DVD-R", FormatFileSize.FormatDecimal(4700000000)), 4707319808);
            }
        }

        public static DiscType DVDPlus
        {
            get
            {
                return new DiscType(String.Format(BaseForm.RM.GetString("{0} Single Layer ({1})"), "DVD+R", FormatFileSize.FormatDecimal(4700000000)), 4700372992);
            }
        }

        public static DiscType CD99
        {
            get
            {
                return new DiscType(String.Format(BaseForm.RM.GetString("CD {0}min ({1})"), "99", FormatFileSize.FormatDecimal(870000000)), 912384000);
            }
        }

        public static DiscType CD90
        {
            get
            {
                return new DiscType(String.Format(BaseForm.RM.GetString("CD {0}min ({1})"), "90", FormatFileSize.FormatDecimal(790000000)), 829440000);
            }
        }

        public static DiscType CD80
        {
            get
            {
                return new DiscType(String.Format(BaseForm.RM.GetString("CD {0}min ({1})"), "80", FormatFileSize.FormatDecimal(700000000)), 737280000);
            }
        }

        public static DiscType CD74
        {
            get
            {
                return new DiscType(String.Format(BaseForm.RM.GetString("CD {0}min ({1})"), "74", FormatFileSize.FormatDecimal(650000000)), 681984000);
            }
        }

        public static DiscType CD63
        {
            get
            {
                return new DiscType(String.Format(BaseForm.RM.GetString("CD {0}min ({1})"), "63", FormatFileSize.FormatDecimal(553000000)), 580608000);
            }
        }

        public static DiscType CD21
        {
            get
            {
                return new DiscType(String.Format(BaseForm.RM.GetString("CD {0}min ({1})"), "21", FormatFileSize.FormatDecimal(185000000)), 193536000);
            }
        }

        public static DiscType HDDVDDL
        {
            get
            {
                return new DiscType(String.Format(BaseForm.RM.GetString("HD-DVD Dual Layer ({0})"), FormatFileSize.FormatDecimal(30000000000)), 33393473536);
            }
        }

        public static DiscType HDDVD
        {
            get
            {
                return new DiscType(String.Format(BaseForm.RM.GetString("HD-DVD Single Layer ({0})"), FormatFileSize.FormatDecimal(15000000000)), 15076554752);
            }
        }

        public static DiscType BDDL
        {
            get
            {
                return new DiscType(String.Format(BaseForm.RM.GetString("Blu-Ray Disc Dual Layer ({0})"), FormatFileSize.FormatDecimal(50000000000)), 50050629632);
            }
        }

        public static DiscType BD
        {
            get
            {
                return new DiscType(String.Format(BaseForm.RM.GetString("Blu-Ray Disc Single Layer ({0})"), FormatFileSize.FormatDecimal(25000000000)), 25025314816);
            }
        }

        #endregion

        #region Properties

        /// <summary>
        /// Translated display string of the disc type.
        /// </summary>
        public string Description
        {
            get { return description; }
            set { description = value; }
        }

        public long SizeInBytes
        {
            get { return sizeInBytes; }
            set { sizeInBytes = value; }
        }

        #endregion

        private string description;
        private long sizeInBytes = 0;

        public DiscType(string description, long sizeInBytes)
        {
            this.description = description;
            this.sizeInBytes = sizeInBytes;
        }

        public override string ToString()
        {
            return this.description;
        }
    }
}
