﻿using System;
using System.Collections.Generic;
using System.Text;
using StarBurn = RocketDivision.StarBurnX;
using System.Runtime.InteropServices;
using RocketDivision.StarBurnX;
using System.IO;

namespace CDBurnerXP.Burning
{
    public class Grabber : StarBurn.GrabberClass
    {
        #region GrabIsoInfo

        public class GrabIsoInfo
        {
            public string FileName;
            public bool IsAudio;
            public int TrackSizeLB = 0;

            public GrabIsoInfo(string fileName, bool isAudio, int trackSizeLB)
            {
                this.FileName = fileName;
                this.IsAudio = isAudio;
                this.TrackSizeLB = trackSizeLB;
            }
        }

        #endregion

        private bool grabWholeDisc = false;
        private int tracksCompleted = 0;
        private int trackCount = 0;

        public delegate void ProgressEventHandler(int tracknumber, bool isNextTrack, int totalPercent, TimeSpan timeRemaining);

        /// <summary>
        /// Fired on every status update of the grabbing process.
        /// </summary>
        public event ProgressEventHandler OnProgressExtended;

        public Grabber()
        {
            this.OnProgress += new StarBurn.IGrabberEvents_OnProgressEventHandler(Grabber_OnProgress);
        }

        public override void GrabDisc(string FileName)
        {
            ReadTrackCount();
            this.grabWholeDisc = true;
            base.GrabDisc(FileName);
        }

        /// <summary>
        /// Since burning MDS images is not supported in all scenarios, we 
        /// have to store a disc as single ISO tracks and burn these to disc.
        /// </summary>
        /// <param name="targetFolder">Folder in which all tracks should be stored.</param>
        public Dictionary<int, GrabIsoInfo> GrabDiscAsMultipleTracks(string targetFolder)
        {
            ReadTrackCount();
            this.grabWholeDisc = true;
            this.FastReadTOC = false;

            int trackCounter = 1;
            Dictionary<int, GrabIsoInfo> result = new Dictionary<int, GrabIsoInfo>();

            Directory.CreateDirectory(targetFolder);

            for (int i = 0; i < this.Drive.DiscInfo.Sessions.Count; i++)
            {
                Session session = this.Drive.DiscInfo.Sessions[i];

                for (int j = 0; j < session.Tracks.Count; j++)
                {
                    string targetFile = Path.Combine(targetFolder, "Track" + trackCounter + ".iso");
                    base.GrabTrackAsData(j+1, targetFile);
                    result.Add(trackCounter, new GrabIsoInfo(targetFile, session.Tracks[j].Audio, session.Tracks[j].Size));
                    trackCounter++;
                }
            }

            return result;
        }

        public override void GrabTrackAsAudio(int nTrackNumber, string FileName)
        {
            ReadTrackCount();
            this.grabWholeDisc = false;
            base.GrabTrackAsAudio(nTrackNumber, FileName);
        }

        public override void GrabDVDVideoDisc(string FileName)
        {
            ReadTrackCount();
            this.grabWholeDisc = true;
            base.GrabDVDVideoDisc(FileName);
        }

        public override void GrabTrackAsData(int nTrackNumber, string FileName)
        {
            ReadTrackCount();
            this.grabWholeDisc = false;
            base.GrabTrackAsData(nTrackNumber, FileName);
        }

        private void ReadTrackCount()
        {
            this.trackCount = 0;
            this.tracksCompleted = 0;

            if (this.Drive == null) return;

            for (int i = 0; i < this.Drive.DiscInfo.Sessions.Count; i++)
            {
                Session session = this.Drive.DiscInfo.Sessions[i];

                for (int j = 0; j < session.Tracks.Count; j++)
                {
                    this.trackCount++;
                }
            }
        }

        private void Grabber_OnProgress(int percent, int timeRemaining)
        {
            bool isNextTrack = false;

            int totalProgress = 0;
            if (this.grabWholeDisc)
            {
                totalProgress = (this.trackCount == 0) ? 0 : Conversion.ToInt((tracksCompleted * 100.0 + percent) / this.trackCount);
            }
            else
            {
                totalProgress = percent;
            }

            // isNextTrack = true for the very first track
            if (percent == 0 && tracksCompleted == 0)
            {
                isNextTrack = true;
            }
            else if (percent == 100 && tracksCompleted < this.trackCount - 1)
            {
                // Increase track completed count only until we reach the last track
                tracksCompleted++;
                isNextTrack = true;
            }

            if (OnProgressExtended != null)
            {
                OnProgressExtended(tracksCompleted + 1, isNextTrack, totalProgress, TimeSpan.FromSeconds(timeRemaining));
            }
        }

        #region Useless COM overrides

        public override RocketDivision.StarBurnX.STARBURN_AUDIO_COMPRESSION AudioCompression
        {
            get
            {
                return base.AudioCompression;
            }
            set
            {
                base.AudioCompression = value;
            }
        }

        public override void Cancel()
        {
            base.Cancel();
        }

        public override RocketDivision.StarBurnX.STARBURN_DISC_READ_MODE_RAW CDRawReadMode
        {
            get
            {
                return base.CDRawReadMode;
            }
            set
            {
                base.CDRawReadMode = value;
            }
        }

        public override bool DisableHardwareErrorCorrection
        {
            get
            {
                return base.DisableHardwareErrorCorrection;
            }
            set
            {
                base.DisableHardwareErrorCorrection = value;
            }
        }

        public override RocketDivision.StarBurnX.Drive Drive
        {
            get
            {
                return base.Drive;
            }
            set
            {
                base.Drive = value;
            }
        }

        public override bool FastReadTOC
        {
            get
            {
                return base.FastReadTOC;
            }
            set
            {
                base.FastReadTOC = value;
            }
        }

        public override bool IgnoreBadBlocks
        {
            get
            {
                return base.IgnoreBadBlocks;
            }
            set
            {
                base.IgnoreBadBlocks = value;
            }
        }

        public override string LastError
        {
            get
            {
                return base.LastError;
            }
        }

        public override RocketDivision.StarBurnX.STARBURN_RESULT_CODES LastErrorCode
        {
            get
            {
                return base.LastErrorCode;
            }
        }

        public override int LastWin32ErrorCode
        {
            get
            {
                return base.LastWin32ErrorCode;
            }
        }

        public override bool LockTray
        {
            get
            {
                return base.LockTray;
            }
            set
            {
                base.LockTray = value;
            }
        }

        public override int NumberOfReadRetries
        {
            get
            {
                return base.NumberOfReadRetries;
            }
            set
            {
                base.NumberOfReadRetries = value;
            }
        }

        public override bool OnBadBlockHitEnabled
        {
            get
            {
                return base.OnBadBlockHitEnabled;
            }
            set
            {
                base.OnBadBlockHitEnabled = value;
            }
        }

        public override int ReadSpeed
        {
            get
            {
                return base.ReadSpeed;
            }
            set
            {
                base.ReadSpeed = value;
            }
        }

        public override int SplitSize
        {
            get
            {
                return base.SplitSize;
            }
            set
            {
                base.SplitSize = value;
            }
        }

        #endregion
    }
}
