﻿using System;
using System.Collections.Generic;
using System.Text;
using StarBurn = RocketDivision.StarBurnX;
using System.Runtime.InteropServices;
using RocketDivision.StarBurnX;

namespace CDBurnerXP.Burning
{
    public class ImageBurner : StarBurn.ImageBurnerClass
    {
        private bool cancel = false;
        private int modifyProgressTotalLB = 0;
        private int modifyProgressLBWritten = 0;
        private int modifyProgressCurrentTrackLB = 0;
        private AudioBurner currentAudioBurner = null;

        public delegate void ProgressEventHandler(int totalPercent, int timeRemaining);

        /// <summary>
        /// Fired on every status update of the grabbing process.
        /// </summary>
        public event ProgressEventHandler OnProgressExtended;

        public ImageBurner()
        {
            this.OnProgress += new RocketDivision.StarBurnX.IImageBurnerEvents_OnProgressEventHandler(ImageBurner_OnProgress);
        }

        private void ImageBurner_OnProgress(int Percent, int TimeRemaining)
        {
            if (OnProgressExtended != null)
            {
                int currentTrackLBWritten = Conversion.ToInt(Percent / 100f * this.modifyProgressCurrentTrackLB);
                int newPercent = Conversion.ToInt(((this.modifyProgressLBWritten + currentTrackLBWritten) / (float)this.modifyProgressTotalLB) * 100);
                OnProgressExtended(newPercent, TimeRemaining);
            }
        }

        /// <summary>
        /// Burns the specified ISO tracks, previously saved by the Grabber class, to a disc.
        /// </summary>
        public void Burn(Dictionary<int, Grabber.GrabIsoInfo> isoTracks)
        {
            this.modifyProgressTotalLB = 0;
            this.modifyProgressCurrentTrackLB = 0;
            this.modifyProgressLBWritten = 0;

            foreach (KeyValuePair<int, Grabber.GrabIsoInfo> track in isoTracks)
            {
                this.modifyProgressTotalLB += track.Value.TrackSizeLB;
            }
            bool finaliseDisc = !this.NextSessionAllowed;

            foreach (KeyValuePair<int, Grabber.GrabIsoInfo> track in isoTracks)
            {
                this.ImageFileName = track.Value.FileName;
                this.Mode = RocketDivision.StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_TAO;
                if (track.Key < isoTracks.Count)
                {
                    this.NextSessionAllowed = true;
                }
                else
                {
                    this.NextSessionAllowed = !finaliseDisc;
                }

                this.modifyProgressCurrentTrackLB = track.Value.TrackSizeLB;
                if (track.Value.IsAudio)
                {
                    AudioBurner burner = new AudioBurner();
                    currentAudioBurner = burner;
                    burner.Drive = this.Drive;
                    burner.Mode = RocketDivision.StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_TAO;
                    // Close session if next track is not audio
                    burner.KeepSessionOpened = isoTracks.ContainsKey(track.Key + 1) && isoTracks[track.Key + 1].IsAudio;
                    burner.NextSessionAllowed = this.NextSessionAllowed;
                    burner.AddFile(track.Value.FileName);
                    burner.WriteSpeed = this.WriteSpeed;
                    burner.OnProgress += new RocketDivision.StarBurnX.IAudioBurnerEvents_OnProgressEventHandler(delegate(int Percent, int TimeRemaining)
                    {
                        ImageBurner_OnProgress(Percent, TimeRemaining);
                    });
                    burner.Burn(false);
                }
                else
                {
                    this.BurnEx(false, false, true);
                }
                this.modifyProgressLBWritten += track.Value.TrackSizeLB;

                if (this.cancel)
                {
                    this.cancel = false;
                    break;
                }
            }
        }

        public override void Burn(bool TestBurn)
        {
            base.Burn(TestBurn);
        }

        public override void BurnEx(bool TestBurn, bool IsMDSImage, bool IsXA)
        {
            try
            {
                base.BurnEx(TestBurn, IsMDSImage, IsXA);
            }
            catch (COMException ex)
            {
                // MDS images: Prevent non SAO write modes.
                if (IsMDSImage && ex.IsError(STARBURN_RESULT_CODES.STARBURN_ERR_WRITEMODE_NOT_SUPPORTED))
                {
                    this.Mode = STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_SAO;
                    base.BurnEx(TestBurn, IsMDSImage, IsXA);
                }
            }
        }

        public override void Cancel()
        {
            this.cancel = true;
            base.Cancel();
            if (this.currentAudioBurner != null)
            {
                this.currentAudioBurner.Cancel();
            }
        }

        public override RocketDivision.StarBurnX.Drive Drive
        {
            get
            {
                return base.Drive;
            }
            set
            {
                base.Drive = value;
            }
        }

        public override string ImageFileName
        {
            get
            {
                return base.ImageFileName;
            }
            set
            {
                base.ImageFileName = value;
            }
        }

        public override string LastError
        {
            get
            {
                return base.LastError;
            }
        }

        public override RocketDivision.StarBurnX.STARBURN_RESULT_CODES LastErrorCode
        {
            get
            {
                return base.LastErrorCode;
            }
        }

        public override int LastWin32ErrorCode
        {
            get
            {
                return base.LastWin32ErrorCode;
            }
        }

        public override bool LockTray
        {
            get
            {
                return base.LockTray;
            }
            set
            {
                base.LockTray = value;
            }
        }

        public override RocketDivision.StarBurnX.STARBURN_WRITE_MODE Mode
        {
            get
            {
                return base.Mode;
            }
            set
            {
                base.Mode = value;
            }
        }

        public override bool NextSessionAllowed
        {
            get
            {
                return base.NextSessionAllowed;
            }
            set
            {
                base.NextSessionAllowed = value;
            }
        }

        public override bool Padding
        {
            get
            {
                return base.Padding;
            }
            set
            {
                base.Padding = value;
            }
        }

        public override int ReadSpeed
        {
            get
            {
                return base.ReadSpeed;
            }
            set
            {
                base.ReadSpeed = value;
            }
        }

        public override void Verify()
        {
            base.Verify();
        }

        public override int WriteSpeed
        {
            get
            {
                return base.WriteSpeed;
            }
            set
            {
                base.WriteSpeed = value;
            }
        }

        public override bool KeepSessionOpened
        {
            get
            {
                return base.KeepSessionOpened;
            }
            set
            {
                base.KeepSessionOpened = value;
            }
        }

        public override bool IsXACDROM
        {
            get
            {
                return base.IsXACDROM;
            }
            set
            {
                base.IsXACDROM = value;
            }
        }
    }
}
