﻿using System;

namespace CDBurnerXP.Burning
{
    public class RemainingCalculator
    {
        private static int m_LastProgress = 0;
        private static TimeSpan m_LastElapsedTime = TimeSpan.Zero;
        private static TimeSpan m_LastRemainingTime = TimeSpan.Zero;
        private static TimeSpan m_InitValue = TimeSpan.Zero;
        private static TimeSpan m_lastChangedElapsedTime = TimeSpan.Zero;
        private static string m_MinutesString = "";
        private static string m_MinuteString = "";
        private static TimeSpan m_ForgetTime = TimeSpan.Zero;
        private static TimeSpan m_Pause = TimeSpan.Zero;
        private static DateTime m_StartPause = DateTime.MinValue;

        public static string MinutesString
        {
            get { return m_MinutesString; }
            set { m_MinutesString = value; }
        }

        public static string MinuteString
        {
            get { return m_MinuteString; }
            set { m_MinuteString = value; }
        }

        public static void Init(long bytes, long bytesPerSecond)
        {
            int seconds = 0;
            if (bytesPerSecond > 0)
            {
                seconds = Conversion.ToInt(bytes / bytesPerSecond);
            }

            Reset();
            // one extra minute
            m_InitValue = new TimeSpan(0, 1, seconds);
        }

        public static void Reset()
        {
            m_LastProgress = 0;
            m_LastElapsedTime = TimeSpan.Zero;
            m_LastRemainingTime = TimeSpan.Zero;
            m_InitValue = TimeSpan.Zero;
            m_lastChangedElapsedTime = TimeSpan.Zero;
            m_ForgetTime = TimeSpan.Zero;
        }

        public static void Forget(TimeSpan duration)
        {
            m_ForgetTime = duration;
        }

        public static string GetRemainingTime(TimeSpan elapsed, int progress)
        {
            if (progress == 0)
            {
                m_Pause = TimeSpan.Zero;
                m_StartPause = DateTime.MinValue;
            }
            // Let's wait a little until the value is more stable
            if (progress < 2 | progress >= 100)
            {
                if (m_LastProgress != progress)
                {
                    m_LastProgress = progress;
                    m_lastChangedElapsedTime = elapsed;
                }
                m_LastElapsedTime = elapsed;
                if (m_InitValue != TimeSpan.Zero && !(progress >= 100))
                {
                    m_LastRemainingTime = m_InitValue;
                }
                else
                {
                    if (progress >= 100)
                    {
                        Reset();
                    }
                    return "-";
                }
            }

            elapsed -= m_ForgetTime;
            if (m_ForgetTime != TimeSpan.Zero)
            {
                progress -= 50;
            }

            // Recalculate only if progress changed or not paused
            if (m_LastProgress != progress && progress > 0)
            {
                m_LastProgress = progress;
                // How much time did we need for progress?
                double onePercentTime = elapsed.TotalMilliseconds / progress;
                // How long took the last 1 percent?
                double onePercentLastTime = elapsed.TotalMilliseconds - m_lastChangedElapsedTime.TotalMilliseconds;
                // Use average
                onePercentTime = (onePercentLastTime * 2 + onePercentTime) / 3;
                try
                {
                    TimeSpan remainingTime = TimeSpan.FromMilliseconds(onePercentTime * (100 - progress));
                    m_LastRemainingTime = new TimeSpan(0, 0, Convert.ToInt32(remainingTime.TotalSeconds));
                }
                catch (OverflowException)
                {
                    // if process takes too long, only show the previous value
                }
                m_lastChangedElapsedTime = elapsed;
                m_LastElapsedTime = elapsed;
            }
            else
            {
                // Give simple difference
                TimeSpan diff = elapsed - m_LastElapsedTime;
                m_LastElapsedTime = elapsed;
                m_LastRemainingTime -= diff;
            }

            if (m_LastRemainingTime.TotalMinutes > 1)
            {
                return string.Format(MinutesString, m_LastRemainingTime.TotalMinutes);
            }
            else
            {
                return string.Format(MinuteString, m_LastRemainingTime.TotalMinutes);
            }
        }
    }
}