﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.ComponentModel;

namespace CDBurnerXP.Controls
{
    /// <summary>
    /// Draws each item in the combo box with an icon.
    /// </summary>
    public class ImageComboBox : ComboBox
    {
        private ImageList imageList = null;
        private Dictionary<int, int> itemImages = new Dictionary<int, int>();

        #region Properties

        /// <summary>
        /// Gets the image list that contains images to be used with the items.
        /// </summary>
        [DefaultValue(null)]
        public ImageList ImageList
        {
            get { return this.imageList; }
            set { this.imageList = value; }
        }

        /// <summary>
        /// Dictionary that associates each item in the combo box with an image in the image list.
        /// </summary>
        public Dictionary<int, int> ItemImages
        {
            get { return this.itemImages; }
        }

        #endregion

        protected override void OnDrawItem(DrawItemEventArgs e)
        {
            base.OnDrawItem(e);

            // Don't trust that this function gets only called with a valid index.
            if (e.Index < 0 || e.Index >= this.Items.Count)
            {
                return;
            }

            e.DrawFocusRectangle();
            e.DrawBackground();

            StringFormat format = new StringFormat();
            format.FormatFlags = StringFormatFlags.NoClip;
            format.LineAlignment = StringAlignment.Center;

            string itemText = this.GetItemText(this.Items[e.Index]);

            Image image = this.itemImages.ContainsKey(e.Index) && this.imageList != null ? this.imageList.Images[this.itemImages[e.Index]] : null;
            int indent = (image == null) ? 0 : image.Width + 3;

            if (this.RightToLeft == RightToLeft.Yes)
            {
                format.Alignment = StringAlignment.Far;
                e.Graphics.DrawString(itemText, this.Font, new SolidBrush(e.ForeColor), new RectangleF((float)e.Bounds.X, (float)e.Bounds.Y, (float)(e.Bounds.Width - indent), (float)e.Bounds.Height), format);
            }
            else
            {
                format.Alignment = StringAlignment.Near;
                if (image != null)
                {
                    e.Graphics.DrawImageUnscaled(image, e.Bounds);
                }
                e.Graphics.DrawString(itemText, this.Font, new SolidBrush(e.ForeColor), new RectangleF((float)(e.Bounds.X + indent), (float)e.Bounds.Y, (float)e.Bounds.Width, (float)e.Bounds.Height), format);
            }
        }

    }
}
