using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using CDBurnerXP.IO;

namespace CDBurnerXP.Controls
{
    public class LogListView : ListView
    {
        public enum LogType
        {
            Information,
            Warning,
            Error
        }

        private static string m_EventColHeader = "Event:";
        private static string m_CopyLog = "&Copy log";
        private ContextMenu cmnuLog = new ContextMenu();
        private static string m_TimeColHeader = "Time:";
        private string m_LastMessage = null;

        #region Properties

        public static string CopyLogText
        {
            set
            {
                m_CopyLog = value;
            }
        }

        public static string EventColHeader
        {
            set
            {
                m_EventColHeader = value;
            }
        }

        public static string TimeColHeader
        {
            set
            {
                m_TimeColHeader = value;
            }
        }

        #endregion


        public LogListView()
            : base()
        {
            Columns.Clear();
            Columns.Add("Icons", "", 26);
            Columns.Add("Time", m_TimeColHeader, 73);
            Columns.Add("Events", m_EventColHeader, this.Width - 73 - 25 - 25);
            // Load images
            this.SmallImageList = new ImageList();
            this.SmallImageList.ColorDepth = ColorDepth.Depth32Bit;
            SmallImageList.Images.Add(System.Drawing.SystemIcons.Information);
            SmallImageList.Images.Add(System.Drawing.SystemIcons.Warning);
            SmallImageList.Images.Add(System.Drawing.SystemIcons.Error);

            DoubleBuffered = true;
            FullRowSelect = true;
            HeaderStyle = ColumnHeaderStyle.Nonclickable;
            View = View.Details;

            ContextMenu = cmnuLog;
            cmnuLog.MenuItems.Add(new MenuItem(m_CopyLog, LogListView_Click));
        }

        private void LogListView_Click(object sender, EventArgs e)
        {
            StringBuilder sb = new StringBuilder();

            foreach (ListViewItem item in this.Items)
            {
                sb.Append(item.SubItems[1].Text);
                sb.Append("\t");
                sb.AppendLine(item.SubItems[2].Text);
            }

            SafeClipboard.SetData(sb.ToString(), true);
        }
        
        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            if (Width > 0 && Columns.ContainsKey("Events"))
            {
                Columns["Events"].Width = this.Width - Columns["Time"].Width - Columns["Icons"].Width - 25;
            }
        }

        public void AddItem(LogType type, string text)
        {
            if (InvokeRequired)
            {
                this.BeginInvoke((MethodInvoker)delegate() { AddItem(type, text); });
                return;
            }

            // Do not add duplicate messages
            if (m_LastMessage == text)
            {
                return;
            }

            ListViewItem item = new ListViewItem("", (int)type);
            item.SubItems.Add(DateTime.Now.ToLongTimeString());
            item.SubItems.Add(text.Replace("\r\n", " ").Replace("\n", " "));
            Items.Add(item);

            m_LastMessage = text;

            item.EnsureVisible();
            Refresh();
        }

        private void InitializeComponent()
        {
            this.cmnuLog = new System.Windows.Forms.ContextMenu();
            this.SuspendLayout();
            this.ResumeLayout(false);

        }
    }
}
