﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CDBurnerXP.Controls;
using System.IO;

namespace CDBurnerXP.Forms
{
    public partial class FileAddErrorsDialog : FileReportBase
    {
        private List<ItemAddError> m_Errors = new List<ItemAddError>();

        public ItemAddError[] Errors
        {
            set
            {
                m_Errors.Clear();
                m_Errors.AddRange(value);
            }
            get
            {
                return m_Errors.ToArray();
            }
        }

        public FileAddErrorsDialog()
        {
            InitializeComponent();

            cName.Text = RM.GetString("Name");
            cDetails.Text = RM.GetString("Details");
            cSourcePath.Text = RM.GetString("Source path");

            cName.AspectGetter = delegate(object x) { return Path.GetFileName(((ItemAddError)x).Path); };
            cName.ImageGetter = delegate(object x)
            {
                ItemAddError error = x as ItemAddError;
                try
                {
                    if (File.Exists(error.Path)) return 2;
                    if (Directory.Exists(error.Path)) return 1;
                }
                catch { }
                return 0;
            };
            cDetails.AspectGetter = delegate(object x) { return ((ItemAddError)x).Message; };
            cSourcePath.AspectGetter = delegate(object x) { return ((ItemAddError)x).Path; };
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            olvErrors.SetObjects(m_Errors);
        }

        /// <summary>
        /// Simply displays the dialog, for your convenience.
        /// </summary>
        /// <param name="errors"></param>
        public static void Show(IWin32Window owner, ItemAddError[] errors)
        {
            // Don't show empty report
            if (errors == null || errors.Length == 0) return;

            using (FileAddErrorsDialog dialog = new FileAddErrorsDialog())
            {
                dialog.Errors = errors;
                dialog.ShowDialog(owner);
            }
        }
    }
}
