using System;
using System.Collections;
using System.Runtime.InteropServices;
using Microsoft.Win32;

namespace CDBurnerXP.Text
{
    /// <summary>
    /// Uses the Windows native function to compare file names.
    /// </summary>
    public sealed class StringLogicalComparer : IComparer
    {
        [DllImport("shlwapi.dll", CharSet = CharSet.Unicode)]
        private static extern int StrCmpLogicalW(string psz1, string psz2);

        private static readonly IComparer _default = new StringLogicalComparer();
        private static bool? winFunctionExists = null;

        private StringLogicalComparer()
        {}

        public static IComparer Default
        {
            get { return _default; }
        }

        public int Compare(object x, object y)
        {
            if (null == x && null == y) return 0;
            if (null == x) return -1;
            if (null == y) return 1;
            if (x is string && y is string)
                return Compare((string)x, (string)y);
            return Comparer.Default.Compare(x, y);
        }

        public static int Compare(string s1, string s2)
        {
            if (winFunctionExists == null)
            {
                // Check if comparison function exists
                IntPtr dll = Kernel32.LoadWin32Library("shlwapi.dll");
                if (dll == null || dll == IntPtr.Zero)
                {
                    winFunctionExists = false;
                }
                else
                {
                    try
                    {
                        IntPtr function = Kernel32.GetProcAddress(dll, "StrCmpLogicalW");
                        winFunctionExists = (function != null && function != IntPtr.Zero);
                    }
                    finally
                    {
                        Kernel32.FreeLibraryEx(dll);
                    }
                }
            }

            if (winFunctionExists.HasValue && winFunctionExists.Value)
            {
                return StrCmpLogicalW(s1, s2);
            }
            else
            {
                return s1.CompareTo(s2);
            }
		}
	}
}
